#include <reg51.h>
#include <stdio.h>    /* stardard i/o definitions          */
#include <math.h>
#include "AL242.h"
#include "AL251.h"

sbit _AL128_LED =P2^6;
extern unsigned char  source;


#define	DATA_COMMAND	0xff
#define	DATA_END		0xff
#define	COMMAND_DELAY	0x80
#define	DELAY_VALUE		0x7f

#define AFE_GAIN_INIT	0x0f

void SignalStatusDisplay(void);

// Initial video decode register 
void InitReg(unsigned char *pcDat)
{
	unsigned char cIndex, cDat;
	while(1)
	{
		if (*pcDat == DATA_COMMAND) 
		{
			if (*((unsigned char *)(pcDat + 1)) == DATA_END)
			{
				break;
			}
			else if (((*((unsigned char *)(pcDat + 1))) & COMMAND_DELAY) == COMMAND_DELAY)
			{
				alDelay((*((unsigned char *)(pcDat + 1))) & DELAY_VALUE);
				pcDat += 2;
			}
			else
			{
//				cIndex = *pcDat++;
//				cDat = *pcDat++;
				// set I2C addres
//				cChip_addr = cDat<<1;
			}
		}
		else
		{
			cIndex = *pcDat++;
			cDat = *pcDat++;
			SetAL242Reg(cIndex,cDat);
		}
	}
}



void SetAl240Standard(unsigned char cStd)
{
	switch(cStd)
	{
		case STD_NTSC_M:
			InitReg(szNTSC_M);
			break;

		case STD_NTSC_443:
			InitReg(szNTSC_443);
			break;

		case STD_PAL_I:
			InitReg(szPAL_I);
			break;

		case STD_PAL_M:
			InitReg(szPAL_M);
			break;

		case STD_PAL_CN:
			InitReg(szPAL_CN);
			break;

		case STD_SECAM:
			InitReg(szSECAM);
			break;

		case STD_PAL_60:
			InitReg(szPAL_60);
			break;

		case STD_BW625:
			InitReg(szPAL_I);
			break;

		case STD_BW525:
			InitReg(szNTSC_M);
			break;

		default:
			InitReg(szNTSC_M);
			break;

	}

}


//=============================================================================
// AL242  video input: NTSC-M 
//=============================================================================

code unsigned char szNTSC_M[]={
0xFE,0xFF,	//new
0x00,0x00,
0x01,0x02,
0x02,0x4F,
0x03,0x00,
0x04,0xDD,
0x07,0x20,
0x0C,0x8a,     
0x0F,0x2C,
0x16,0x72,
0x18,0x2D,
0x19,0xD1,
0x1A,0x74,
0x1B,0x5D,
0x1C,0x2B,
0x1D,0x33,
0x1E,0x33,
0x1F,0x33,
0x2D,0x48,
0x2E,0x82,
0x30,0x22,
0x31,0x61,
0x34,0x6e,
0x69,0x48,
0x6A,0x50,
0x82,0x42,
0x83,0x6e,
0xBA,0x18,
0xBB,0x35,
0xBC,0x2c,
0xBF,0x03,
0xc0,0xd4,
0xC1,0x1B,
0xC2,0x20,
0xC4,0x07,
0xD4,0x3C,
0xD5,0x05,     //05-->04   kim0922       reduce  NTSC/S-VIDEO  screen flicker
0x3F,0x01,	// software reset
0x3F,0x00,
0xFF,0xFF};



//=============================================================================
// AL242  video input: NTSC-4.43 
//=============================================================================
code unsigned char szNTSC_443[]={
//0xFF,0x2A/2,
0xFE,0xFF,
0x00,0x00,
0x01,0x02,
0x02,0x4F,
0x03,0x03,
0x04,0xDD,
0x07,0x20,   
0x0C,0x8a,  
0x0F,0x2C,
0x16,0x72,
0x18,0x38,
0x19,0xC0,
0x1A,0x14,
0x1B,0xF8,
0x1C,0x2B,
0x1D,0x33,
0x1E,0x33,
0x1F,0x33,
0x2D,0x50,
0x2E,0x7E,
0x30,0x13,
0x31,0x61,
0x34,0x6E,
0x69,0x48,
0x6A,0x50,
0x82,0x22,
0x83,0x6E,
0xBA,0x18,
0xBB,0x35,
0xBC,0x3C,
0xBF,0x03,
0xC1,0x14,
0xC2,0x20,
0xC4,0x07,
0xD4,0x3C,
0xD5,0x05,     //05-->04   kim0922       reduce  NTSC/S-VIDEO  screen flicker
0x3F,0x01,	// software reset
0x3F,0x00,
0xFF,0xFF};


//=============================================================================
// AL242  video input: PAL-I 
//=============================================================================
code unsigned char szPAL_I[]={

0xFE,0xFF,		//NEW
0x00,0x32,
0x01,0x02,	//00
0x02,0x4F,
0x03,0x00,
0x04,0xDC,	
0x07,0x20,
0x0C,0x67,
0x0F,0x2C,
0x16,0x72,
0x18,0x38,	
0x19,0xC0,	
0x1A,0x14,	
0x1B,0xF8,	
0x1C,0x2B,
0x1D,0x33,
0x1E,0x33,
0x1F,0x33,
0x2D,0x50,	//46
0x2E,0x88,	
0x30,0x0C,	//13
0x31,0xC1,	
0x34,0x6E,	//00
0x69,0x48,
0x6A,0x50,
0x82,0x22,	//42//52
0x83,0x6E,	//6F
0xBA,0x18,
0xBB,0x35,
0xBC,0x3C,	// 2B
0xBF,0x03,
0xC0,0xD4,	
0xC1,0x1B,	// 1B
0xC2,0x20,
0xC4,0x0A,	//07
0xD4,0x3C,
0xD5,0x05,	//04
0x3F,0x01,
0x3F,0x00,
0xFF,0xFF
/*
0xFE,0xFF,		//OLD
0x00,0x32,
0x01,0x00,        // kim0926
0x02,0x4F,
0x03,0x02,        //  06h-->00h-->02h  kim0927    reduce ripple  
0x04,0xDC,
0x07,0x20,
0x0C,0x67,
0x0F,0x2C,
0x18,0x38,
0x19,0xC0,
0x1A,0x14,
0x1B,0xF8,
0x1C,0x2B,
0x1D,0x33,
0x1E,0x33,
0x1F,0x33,
0x2D,0x50, 
0x2E,0x88,
0x30,0x0c,
0x31,0xC1,
0x34,0x00,
0x69,0x48,
0x6A,0x50,
0x82,0x02,   //  font tow  kim0923
0x83,0x6F,
0xBA,0x18,
0xBB,0x35,
0xBC,0x2B,
0xBF,0x03,
0xC1,0x1b,
0xC2,0x20,
0xC4,0x0A,
0xD4,0x3C,
0xD5,0x04,
0x3F,0x01,	// software reset
0x3F,0x00,
0xFF,0xFF*/};

//=============================================================================
// AL242  video input: PAL-M 
//=============================================================================
code unsigned char szPAL_M[]={
0xFE,0xFF,
0x00,0x04,
0x01,0x02,
0x02,0x4F,
0x03,0x02,
0x04,0xDD,
0x07,0x20,
0x0C,0x67,
0x0F,0x2C,
0x18,0x2D,
0x19,0xC4,
0x1A,0x90,
0x1B,0x50,
0x1C,0x2B,
0x1D,0x33,
0x1E,0x33,
0x1F,0x33,
0x2D,0x55,		//53//46
0x2E,0x82,
0x30,0x13,
0x31,0x61,
0x34,0x6E,
0x69,0x48,
0x6A,0x50,
0x82,0x42,
0x83,0x6E,
0xBA,0x18,
0xBB,0x35,
0xBC,0x3C,
0xBF,0x03,
0xC1,0x1B,
0xC2,0x20,
0xC4,0x07,
0xD4,0x3C,
0xD5,0x05,     //05-->04   kim0922       reduce  NTSC/S-VIDEO  screen flicker
0x3F,0x01,	// software reset
0x3F,0x00,
0xFF,0xFF};

//=============================================================================
// AL242  video input: PAL-CN 
//=============================================================================
code unsigned char szPAL_CN[]={
0xFE,0xFF,
0x00,0x36,
0x01,0x02,
0x02,0x4F,
0x03,0x02,
0x04,0xDC,
0x07,0x20,
0x0C,0x67,
0x0F,0x2C,
0x18,0x2D,
0x19,0xD9,
0x1A,0xAE,
0x1B,0x92,
0x1C,0x2B,
0x1D,0x33,
0x1E,0x33,
0x1F,0x33,
0x2D,0x45,
0x2E,0x8A,
0x30,0x0c,
0x31,0xC1,
0x34,0x6E,
0x69,0x48,
0x6A,0x50,
0x82,0x42,
0x83,0x6E,
0xBA,0x18,
0xBB,0x35,
0xBC,0x2B,
0xBF,0x03,
0xC1,0x14,
0xC2,0x20,
0xC4,0x0A,
0xD4,0x3C,
0xD5,0x05,
0x3F,0x01,	// software reset
0x3F,0x00,
0xFF,0xFF};

//=============================================================================
// AL242  video input: SECAM 
//=============================================================================
code unsigned char szSECAM[]={
0xFE,0xFF,
0x00,0x38,
0x01,0x02,
0x02,0x4F,
0x03,0x00,
0x04,0xDC,
0x07,0x20,
0x0C,0x67,
0x0F,0x25,
0x16,0x72,
0x18,0x36,
0x19,0xDC,
0x1A,0x5D,
0x1B,0x63,
0x1C,0x2B,
0x1D,0x33,
0x1E,0x33,
0x1F,0x33,
0x2D,0x68, //for secam
0x2E,0x84,
0x30,0x0c,
0x31,0xC1,
0x34,0x6E,
0x69,0x48,
0x6A,0x50,
0x82,0x22,
0x83,0x6F,		//EF
0xBA,0x18,
0xBB,0x35,
0xBC,0x3C,
0xBF,0x03,
0xC1,0x14,
0xC2,0x20,
0xC4,0x0A,
0xD4,0x3C,
0xD5,0x05,
0x3F,0x01,	// software reset
0x3F,0x00,
0xFF,0xFF};

//=============================================================================
// AL242  video input: PAL-60 
//=============================================================================
code unsigned char szPAL_60[] = {
0xFE,0xFF,
0x00,0x02,
0x01,0x02,
0x02,0x4F,
0x03,0x02,
0x04,0xDC,
0x07,0x20,
0x0C,0x67,
0x0F,0x2C,
0x18,0x38,
0x19,0xC0,
0x1A,0x14,
0x1B,0xF8,
0x1C,0x2B,
0x1D,0x33,
0x1E,0x33,
0x1F,0x33,
0x2D,0x50,
0x2E,0x82,
0x30,0x22,
0x31,0x61,
0x34,0x6E,
0x69,0x48,
0x6A,0x50,
0x82,0x42,
0x83,0x6E,
0xBA,0x18,
0xBB,0x35,
0xBC,0x3C,
0xBF,0x03,
0xC1,0x1B,
0xC2,0x20,
0xC4,0x07,
0xD4,0x3C,
0xD5,0x05,
0x3F,0x01,	// software reset
0x3F,0x00,
0xFF,0xFF};

bit	bSStNoSignal;
unsigned char signaldetectcount;
bdata unsigned char cSignalStatus;
sbit	bSStBwSignal	= cSignalStatus ^ 0;
sbit	bSStHvLock  	= cSignalStatus ^ 1;
sbit	bSSt625line 	= cSignalStatus ^ 2;
sbit	bSStChromaLock 	= cSignalStatus ^ 3;

sbit	bSStPal       	= cSignalStatus ^ 4;
sbit	bSStSecam     	= cSignalStatus ^ 5;
sbit	bSStFcMore    	= cSignalStatus ^ 6;
sbit	bSStFcLess    	= cSignalStatus ^ 7;

bit		bSStLowSignal;

#define AL242REG_SIGNAL_STATUS1		0x3a
#define AL242REG_SIGNAL_STATUS2		0x3b
#define AL242REG_SIGNAL_STATUS3		0x3c
#define AL242REG_LUMA_AGC_HI		0x78
#define AL242REG_LUMA_AGC_LO		0x79
#define AL242REG_CHROMA_AGC_HI		0x7b
#define AL242REG_CHROMA_AGC_LO		0x7c
#define AL242REG_CHROMA_FREQ_ST		0x7d

#define AL242REG_AFE_GAIN        	0xd5

#define AFE_GAIN_X05             	0x00
#define AFE_GAIN_X1             	0x05
#define AFE_GAIN_X2             	0x0a
#define AFE_GAIN_X4             	0x0f

// AFE gain change threshold
#define AFE_GAIN_X05_INC_THD      	0x60
#define AFE_GAIN_X05_DEC_THD      	0x20

#define AFE_GAIN_X1_INC_THD      	0x60
#define AFE_GAIN_X1_DEC_THD      	0x20

#define AFE_GAIN_X2_INC_THD      	0x60
#define AFE_GAIN_X2_DEC_THD      	0x20

#define AFE_GAIN_X4_INC_THD      	0xb8
#define AFE_GAIN_X4_DEC_THD      	0x20

#define	LOW_LEVEL_SIGNAL_THD      	0x05	// Reg78's value when RegD5 = 0x0f


#define	BWSIGNAL_THRESHOLD  		7000 //7000 // THD=7000 can't detect 80mV NTSC443
#define	FCMORE_THD					232
#define	FCLESS_THD					25

#define	CHROMA_FREQ_TOLERANCE    	0x05	// Reg7D change tolerance
#define	REPEAT_READ_COUNT_MAX     	4

// Define wait stable delay time
#define	WAIT_TIME_PRE_625MODE1	2
#define	WAIT_TIME_PRE_625MODE2	2
#define	WAIT_TIME_PRE_525MODE1	2
#define	WAIT_TIME_PRE_525MODE2	2
#define	WAIT_TIME_PRE_525MODE3	2
#define	WAIT_TIME_DETECT_DONE	2
#define   WAIT_STABLE_COUNT			2
//#define	WAIT_TIME_MODE_UNLOCK	10

#define	WAIT_TIME_PAL_M    	2

#define	WAIT_STABLE_TIME	2

enum	DETECT_STATUS
{
	NO_SIGNAL,
	HV_LOCK,
	PRE_DETECT_525MODE1,
	PRE_DETECT_525MODE2,
	PRE_DETECT_525MODE3,
	PRE_DETECT_625MODE1,
	PRE_DETECT_625MODE2,
	DETECT_DONE,
	MODE_UNLOCK
};
/*
code char *spDetStStr[] = {
	"NO_SIGNAL",
	"HV_LOCK",
	"PRE_DETECT_525MODE1",
	"PRE_DETECT_525MODE2",
	"PRE_DETECT_525MODE3",
	"PRE_DETECT_625MODE1",
	"PRE_DETECT_626MODE2",
	"DETECT_DONE",
	"MODE_UNLOCK"
};

code char *spStdStr[] = {
	"STD_NTSC_M",
	"STD_NTSC_443",
	"STD_PAL_I",
	"STD_PAL_M",
	"STD_PAL_CN",
	"STD_SECAM",
	"STD_PAL_60",
	"STD_BW625",
	"STD_BW525",
	"STD_NTSC_J",
	"STD_PAL_N",
	"STD_UNKNOWN"
};
*/
bit	bWaitStable;
bit	bUpdateStandard;

unsigned char cPreSignalStatus;
unsigned char cWaitStableCount;
unsigned char cDetectStatus = NO_SIGNAL;
unsigned char cPreDetectStatus;
unsigned char cCurStandard = STD_NTSC_M;
unsigned char cNewStandard = STD_NTSC_M;


#define	NOSIGNAL_THD	2
unsigned char cNoSignalCount;

void	GetSignalStatus(void)
{
	unsigned char cData;
	unsigned char cTemp1;
	//unsigned char cTemp2;
	unsigned int  iTemp1;


	cData=GetAL242Reg(AL242REG_SIGNAL_STATUS1);

	// bSStNoSignal 	= (cData & 0x01) ? TRUE : FALSE;			
	if (cData & 0x01)
	{
		if (cNoSignalCount > NOSIGNAL_THD)
		{
			bSStNoSignal = TRUE;
			cNoSignalCount = 0;
		}
		else
		{
			cNoSignalCount++;
		}
	}
	else
	{
		bSStNoSignal = FALSE;
		cNoSignalCount = 0;
	}

	bSStHvLock   	= ((cData & 0x06) == 0x06) ? TRUE : FALSE;			
	bSStChromaLock 	= (cData & 0x08) ? TRUE : FALSE;			

	cData=GetAL242Reg(AL242REG_SIGNAL_STATUS3);

	bSStPal     	= (cData & 0x01) ? TRUE : FALSE;			
	bSStSecam   	= (cData & 0x02) ? TRUE : FALSE;			
	bSSt625line    	= (cData & 0x04) ? TRUE : FALSE;			

	// Repeat Read Register to filter out jitter value
	iTemp1 = REPEAT_READ_COUNT_MAX;
	while( iTemp1--)
	{
		cData=GetAL242Reg(AL242REG_CHROMA_FREQ_ST);
		cTemp1 = cData;
		cData=GetAL242Reg(AL242REG_CHROMA_FREQ_ST);
		if ( cabs((signed)cData - (signed)cTemp1) < CHROMA_FREQ_TOLERANCE)
			break;
	}


	bSStFcMore   	= (cData >=128) && (cData < FCMORE_THD) ? TRUE : FALSE;			
	bSStFcLess   	= (cData <128) && (cData > FCLESS_THD) ? TRUE : FALSE;			


	// Use AFE gain, HAGC and CAGC to decide pure Black & White signal

	// Read 16bit register value, maybe generate read error 
	// while low byte's carry add to high byte. 
	// register value toggle between 02ff to 0300,
	// read result will generate 0200, 03ff error value

	cData=GetAL242Reg(AL242REG_CHROMA_AGC_HI);
	cTemp1 = cData<<3;

	cData=GetAL242Reg(AL242REG_CHROMA_AGC_LO);
	cTemp1 += cData>>5;

	iTemp1 = cTemp1;

	cData=GetAL242Reg(AL242REG_LUMA_AGC_HI);
	cTemp1 = cData<<4;

	cData=GetAL242Reg(AL242REG_LUMA_AGC_LO);
	cTemp1 += cData>>4;

	iTemp1 *= cTemp1;

	cData=GetAL242Reg(AL242REG_AFE_GAIN);
	cData &= 0x03;

	iTemp1 <<= cData;

	bSStBwSignal 	= (iTemp1 > BWSIGNAL_THRESHOLD) ? TRUE : FALSE;			

	cData=GetAL242Reg(AL242REG_AFE_GAIN);
	cTemp1=GetAL242Reg(AL242REG_LUMA_AGC_HI);
	bSStLowSignal =  (cData == AFE_GAIN_X4) && (cTemp1 > LOW_LEVEL_SIGNAL_THD) ? TRUE : FALSE;	

	if (bSStBwSignal || bSStNoSignal)
	{
		bSStFcMore = FALSE;
		bSStFcLess = FALSE;
	}
}


/*
void SignalStatusDisplay(void)
{

	if (cPreSignalStatus != cSignalStatus)
	{
		printf("FcL,FcM,SECAM,PAL, CHLK,625,HvLK,B&W: %02x\n", (int)cSignalStatus);
		cPreSignalStatus = cSignalStatus;
	}
}

void DetectStatusDisplay(void)
{
	if (cPreDetectStatus != cDetectStatus)
	{
		printf("Detect Status: %s\n", spDetStStr[cDetectStatus]);
		if ((cDetectStatus != NO_SIGNAL) && (bSStNoSignal))
			printf( "-- No Signal\n ");
		printf("-- FcL,FcM,SECAM,PAL, CHLK,625,HvLK,B&W: %02x\n", (int)cSignalStatus);
		cPreDetectStatus = cDetectStatus;
	}
}
*/

void SignalStatusDisplay(void)
{
//	printf("FcL,FcM,SECAM,PAL, CHROMALK,625,HvLK,B&W: %02x\n", (int)cSignalStatus);
	if (cPreSignalStatus != cSignalStatus)
	{
		cPreSignalStatus = cSignalStatus;
		signaldetectcount = 0;
		cWaitStableCount =  WAIT_STABLE_TIME;
		bWaitStable = 1;		
	}
	else
	{
		if( cCurStandard== STD_SECAM) 
		{
			bWaitStable = 0;
			cWaitStableCount = 0;
		}
		else
		{
			if(signaldetectcount > WAIT_STABLE_COUNT) 	
			{
				bWaitStable = 0;
				cWaitStableCount = 0;
			}
			else
			{
				signaldetectcount++;
			}
		}
	}	
}

char CheckModeLock(void)
{
	if (!bSStHvLock || bSStNoSignal)
	{
		return FALSE;
	}
	else
	{
		switch(cNewStandard)
		{
			case STD_NTSC_M:
			case STD_NTSC_J:
				if (bSStLowSignal)
					return	(!bSSt625line ) ? TRUE : FALSE;
				else
					return	(!bSSt625line && bSStChromaLock && !bSStFcMore && !bSStFcLess && !bSStPal) ? TRUE : FALSE;

			case STD_NTSC_443:
				return	(!bSSt625line && bSStChromaLock && !bSStFcMore && !bSStFcLess && !bSStPal) ? TRUE : FALSE;

			case STD_BW525:
				return	(!bSSt625line && bSStBwSignal ) ? TRUE : FALSE;

			case STD_PAL_60:
				return	(!bSSt625line && bSStChromaLock && !bSStFcMore && !bSStFcLess && bSStPal) ? TRUE : FALSE;

			case STD_PAL_M:
				return	(!bSSt625line && !bSStFcMore && !bSStFcLess && bSStPal) ? TRUE : FALSE;



			case STD_PAL_I:
				if (bSStLowSignal)
					return	(bSSt625line ) ? TRUE : FALSE;
				else
					return	(bSSt625line && bSStChromaLock && !bSStFcMore && !bSStFcLess && bSStPal) ? TRUE : FALSE;

			case STD_PAL_N:
				return	(bSSt625line && bSStChromaLock && !bSStFcMore && !bSStFcLess && bSStPal) ? TRUE : FALSE;

			case STD_PAL_CN:
				//return	(bSSt625line && bSStChromaLock && ( bSStPal || bSStSecam)) ? TRUE : FALSE;
				return	(bSSt625line && bSStChromaLock ) ? TRUE : FALSE;

			case STD_SECAM:
				return	(bSSt625line && bSStChromaLock && bSStSecam ) ? TRUE : FALSE;

			case STD_BW625:
				return	(bSSt625line && bSStBwSignal ) ? TRUE : FALSE;



			default:
				return	FALSE;

		}
	}
}
/*
//
// Auto adjust Analog Front End Gain
//
void AfeAutoGainControl(void)
{
	unsigned char cData;
	unsigned char cHgain;
	unsigned char cPreGain;

	cData=GetAL242Reg(AL242REG_LUMA_AGC_HI);
	cHgain = cData << 4;

	cData=GetAL242Reg(AL242REG_LUMA_AGC_LO);
	cHgain += (cData >> 4);
	
	cData=GetAL242Reg(AL242REG_AFE_GAIN);

	cData &= 0x0f;

	cPreGain = cData;

	switch (cData)
	{
		case AFE_GAIN_X05:
			if (cHgain > AFE_GAIN_X05_INC_THD)
			{
				cData = AFE_GAIN_X1;
			}
			else if (cHgain <= AFE_GAIN_X05_DEC_THD)
			{
//				printf("AFE gain MIN, and input signal too higher!\n");
			}
			break;

		case AFE_GAIN_X1:
			if (cHgain > AFE_GAIN_X1_INC_THD)
			{
				cData = AFE_GAIN_X2;
			}
			else if (cHgain <= AFE_GAIN_X1_DEC_THD)
			{
				cData = AFE_GAIN_X05;
			}
			break;

		case AFE_GAIN_X2:
			if (cHgain > AFE_GAIN_X2_INC_THD)
			{
				cData = AFE_GAIN_X4;
			}
			else if (cHgain <= AFE_GAIN_X2_DEC_THD)
			{
				cData = AFE_GAIN_X1;
			}

			break;

		case AFE_GAIN_X4:
			if (cHgain > AFE_GAIN_X4_INC_THD)
			{
//				printf("AFE gain MAX, and input signal too lower!\n");
			}
			else if (cHgain <= AFE_GAIN_X4_DEC_THD)
			{
				cData = AFE_GAIN_X2;
			}

			break;

		default:
			cData = AFE_GAIN_X1;
	}

	if (cPreGain != cData)
	{
		SetAL242Reg(AL242REG_AFE_GAIN, cData);
//		printf("AFE gain change to %02x \n", (unsigned int) cData);
	}

}
*/

void AfeAutoGainControl(void)
{
	unsigned char cData;
	unsigned char cHgain;
	unsigned char cPreGain;

    	cData=GetAL242Reg(AL242REG_LUMA_AGC_HI);
	cHgain = cData << 4;

    	cData=GetAL242Reg(AL242REG_LUMA_AGC_LO);
	cHgain += (cData >> 4);

    	cData=GetAL242Reg(AL242REG_AFE_GAIN);

	cData &= 0x0f;

	cPreGain = cData;

	switch (cData)
	{
		case AFE_GAIN_X05:
			if (cHgain > AFE_GAIN_X05_INC_THD)
			{
				cData = AFE_GAIN_X1;
			}
			break;

		case AFE_GAIN_X1:
			if (cHgain > AFE_GAIN_X1_INC_THD)
			{
				cData = AFE_GAIN_X2;
			}
			else if (cHgain <= AFE_GAIN_X1_DEC_THD)
			{
				cData = AFE_GAIN_X05;
			}
			break;

		case AFE_GAIN_X2:
			if (cHgain > AFE_GAIN_X2_INC_THD)
			{
				cData = AFE_GAIN_X4;
			}
			else if (cHgain <= AFE_GAIN_X2_DEC_THD)
			{
				cData = AFE_GAIN_X1;
			}

			break;

		case AFE_GAIN_X4:
			if (cHgain <= AFE_GAIN_X4_DEC_THD)
			{
				cData = AFE_GAIN_X2;
			}

			break;

		default:
			cData = AFE_GAIN_X1;
	}

	if (cPreGain != cData)
	{
	        SetAL242Reg(AL242REG_AFE_GAIN,cData); 
 	}
}



void StandardDetect(void)
{
	unsigned char cDat,cDat1;
	
	AfeAutoGainControl();
	GetSignalStatus();

	SignalStatusDisplay();

	

	if (bWaitStable)
	{
		if (cWaitStableCount)
		{
			cWaitStableCount--;
			return;
		}
		else
		{
			bWaitStable = FALSE;
		}		

	}
	cDat=GetAL242Reg(0x3b);
    	cDat=(cDat&0x01)<<5;
	cDat1=GetAL242Reg(0x39);
    	cDat1=(cDat1&0xdf)|cDat;
    	SetAL242Reg(0x39, cDat1);

	switch (cDetectStatus)
	{
		case NO_SIGNAL:
//			printf("no signal\n");
			if (bSStHvLock)
			{
				cDetectStatus = HV_LOCK;
			}
			break;

		case HV_LOCK:
//			printf("hv lock\n");
			if (bSStHvLock)
			{
				if (!bSStLowSignal)
				{
					if (bSSt625line)
					{
					    	cDetectStatus = PRE_DETECT_625MODE1;
	                    			cNewStandard = STD_PAL_I;
						bUpdateStandard = TRUE;
						bWaitStable = TRUE;
						cWaitStableCount =  WAIT_TIME_PRE_625MODE1;
					}
					else
					{
						cDetectStatus = PRE_DETECT_525MODE1;
						cNewStandard = STD_PAL_M;
						bUpdateStandard = TRUE;
	
						bWaitStable = TRUE;
						cWaitStableCount =  WAIT_TIME_PRE_525MODE1;
					}
				}
				else
				{
					if (bSSt625line)
					{
						cDetectStatus = DETECT_DONE;
	
						cNewStandard = STD_PAL_I;
						bUpdateStandard = TRUE;
	
						bWaitStable = TRUE;
						cWaitStableCount =  WAIT_TIME_PRE_625MODE1;
					}
					else
					{
						cDetectStatus = DETECT_DONE;
	
						cNewStandard = STD_NTSC_M;
						bUpdateStandard = TRUE;
	
						bWaitStable = TRUE;
						cWaitStableCount =  WAIT_TIME_PRE_525MODE1;
					}
				}
			}
			else 
			{
				cDetectStatus = MODE_UNLOCK;
			}
			break;

		case PRE_DETECT_525MODE1:
//			printf("pre detect 525mode1\n");
			if (!bSSt625line && bSStBwSignal)
			{
				cNewStandard = STD_BW525;
				bUpdateStandard = TRUE;
				cDetectStatus = DETECT_DONE;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_DETECT_DONE;
			}
			else if (!bSSt625line && bSStChromaLock && !bSStFcMore && !bSStFcLess && bSStPal)
			{
				cNewStandard = STD_PAL_M;
				cDetectStatus = DETECT_DONE;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_DETECT_DONE;
			}
			else if (!bSSt625line && !bSStPal && !bSStFcMore && !bSStFcLess)
			{
				cNewStandard = STD_NTSC_M;
				bUpdateStandard = TRUE;

				cDetectStatus = DETECT_DONE;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_PAL_M;
			}

			else if (!bSSt625line && bSStFcMore)
			{
				cNewStandard = STD_PAL_60;
				bUpdateStandard = TRUE;

				cDetectStatus = PRE_DETECT_525MODE2;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_PRE_525MODE2;
			}
			else if (bSSt625line && bSStHvLock)
			{
				cNewStandard = STD_PAL_I;
				bUpdateStandard = TRUE;

				cDetectStatus = PRE_DETECT_625MODE1;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_PRE_625MODE1;
			}
			else 
			{
				cDetectStatus = MODE_UNLOCK;
			}
			break;

		case PRE_DETECT_525MODE2:
//			printf("pre detect 525 mode2\n");
			if (!bSSt625line && !bSStFcMore && !bSStFcLess && bSStPal && bSStChromaLock)
			{
				cNewStandard = STD_PAL_60;
				bUpdateStandard = TRUE;
				cDetectStatus = DETECT_DONE;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_DETECT_DONE;
			}
			else if (!bSSt625line && !bSStFcMore && !bSStFcLess && !bSStPal)
			{
				cNewStandard = STD_NTSC_443;
				bUpdateStandard = TRUE;

				cDetectStatus = PRE_DETECT_525MODE3;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_PRE_525MODE3;
			}

			else if (bSSt625line && bSStHvLock)
			{
				cNewStandard = STD_PAL_I;
				bUpdateStandard = TRUE;
				cDetectStatus = PRE_DETECT_625MODE1;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_PRE_625MODE1;
			}
			else 
			{
				cDetectStatus = MODE_UNLOCK;
			}

			break;

		case PRE_DETECT_525MODE3:
//			printf("pre detect 525 mode3\n");			
			if (!bSSt625line && !bSStFcMore && !bSStFcLess && !bSStPal && bSStChromaLock)
			{
				cNewStandard = STD_NTSC_443;
				cDetectStatus = DETECT_DONE;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_DETECT_DONE;
			}

			else if (bSSt625line && bSStHvLock)
			{
				cNewStandard = STD_PAL_I;
				bUpdateStandard = TRUE;
				cDetectStatus = PRE_DETECT_625MODE1;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_PRE_625MODE1;
			}
			else 
			{
				cDetectStatus = MODE_UNLOCK;	
			}

			break;

		case PRE_DETECT_625MODE1:
//			printf("pre detect 625 mode1\n");			
			if (bSSt625line && bSStBwSignal)
			{
				cNewStandard = STD_BW625;
				cDetectStatus = DETECT_DONE;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_DETECT_DONE;
			}
			else if (bSSt625line && bSStChromaLock && !bSStFcMore && !bSStFcLess && bSStPal)
			{
				cNewStandard = STD_PAL_I;
				cDetectStatus = DETECT_DONE;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_DETECT_DONE;
			}
			else if (bSSt625line && bSStSecam)
			{
				cNewStandard = STD_SECAM;
				bUpdateStandard = TRUE;

				cDetectStatus = DETECT_DONE;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_DETECT_DONE;
			}

			else if (bSSt625line && bSStFcLess)
			{
				cNewStandard = STD_PAL_CN;
				bUpdateStandard = TRUE;

				cDetectStatus = DETECT_DONE;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_DETECT_DONE;
			}
			else if (!bSSt625line && bSStHvLock)
			{
				cNewStandard = STD_PAL_M;
				bUpdateStandard = TRUE;

				cDetectStatus = PRE_DETECT_525MODE1;
				bWaitStable = TRUE;
				cWaitStableCount = WAIT_TIME_PRE_525MODE1;
			}
			else 
			{
				cDetectStatus = MODE_UNLOCK;	
			}

			break;

		case DETECT_DONE:
//			printf("detect done\n");
			if(CheckModeLock())
			{
				cDetectStatus = DETECT_DONE;
			}
			else
			{
				cDetectStatus = MODE_UNLOCK;
			}
			break;

		case MODE_UNLOCK:
//			printf("mode unlock\n");
			if (bSStHvLock)
			{
				cDetectStatus = HV_LOCK;
			}
			else if (bSStNoSignal)
			{
				cDetectStatus = NO_SIGNAL;
			}

			break;

		default:
            		break;

	}

//	DetectStatusDisplay();
}



void UpdateStandard(void)
{
	if (bUpdateStandard) 
	{
		//bWaitStable = 1;
		//cWaitStableCount = WAIT_STABLE_TIME;
//		printf("====");
		SetAl240Standard(cNewStandard);
//		_AL128_LED=~_AL128_LED;
//		if (cNewStandard < STD_UNKNOWN)
//			printf("%s\n", spStdStr[cNewStandard]);
//		else
//			printf("Unknow mode\n");

		bUpdateStandard = 0;

//		if (bSStLowSignal)
//			printf(" Low Level Signal !!\n");
			
		GetSignalStatus();
//		printf("-- FcL,FcM,SECAM,PAL, CHLK,625,HvLK,B&W: %02x\n", (int)cSignalStatus);

	}

	if (cNewStandard != cCurStandard)
	{
//		if (cNewStandard < STD_UNKNOWN)
//			printf("New STD:%s\n", spStdStr[cNewStandard]);
//		else
//			printf("Unknow mode\n");

		cCurStandard = cNewStandard ;
	}
}


// ********************************************************************************
// Function:   InputChannelSelectAL242;                  
// Description:  Switching AL242 decoder input channel    				 		    
// Input parameters:    _InputChannelX:
//   					       0: _CVBS0 
//    					1: _SVideo_0
// Output parameters:  None.
// Return values:     _Input been assign=_InputChannelX.
// ******************************************************************************** 
unsigned char InputChannelSelectAL242(unsigned char  _InputChannelX)
{ 
	unsigned char _dx;
//	SetAL242OutputMode(1);
	if (  _InputChannelX==0)
		{                                                         //_CVBS0:
//					SetAL242Reg(0x07,0x60);	

//					_dx=GetAL242Reg(0xb2);
//					_dx&=0xdf;
//					SetAL242Reg(0xb2,_dx);			

//					_dx=GetAL242Reg(0x00);
//					_dx&=0xfe;
//					SetAL242Reg(0x00,_dx);
					
					SetAL242Reg(0xb7,0);
							   					 
//					_dx=GetAL242Reg(0x01);
//					_dx&=0x3f;	 
//					_dx|=0x02;
//					SetAL242Reg(0x01,_dx);
					
//					SetAl240filter(cNewStandard);
//					_dx=GetAL242Reg(0x03);
//					_dx&=0xf8;
//					_dx|=0x00;
//					SetAL242Reg(0x03,_dx);
//					SetAL242Reg(0x08, 0xc0);
		}				
  
		else 
			{
			//_SVideo0:
 //					SetAL242Reg(0x07,0x60);
					
//					_dx=GetAL242Reg(0xb2);
//					_dx&=0xdf;
//					SetAL242Reg(0xb2,_dx);

					_dx=GetAL242Reg(0x00);
					_dx|=0x01;
					SetAL242Reg(0x00,_dx);
 
 					SetAL242Reg(0xb7,5);
 	 
//					_dx=GetAL242Reg(0x01);					
//					_dx&=0x3d;
//					SetAL242Reg(0x01,_dx);
 
//       				_dx=GetAL242Reg(0x03);
//					_dx&=0xf8;
//					_dx|=0x03;
					SetAL242Reg(0x03,0x03);
//					SetAL242Reg(0x08, 0x90);
			}	
		return(_InputChannelX);
}


// ********************************************************************************
// Function:    AutoDetectProgress;                  
// Description:  detect AL242 current video standard input then used to setup AL242 
//               video stand config.     				 		    
// Input parameters:   None.
// Output parameters:  _VideoFormatDetect: detect input video mode.
// Return values:      None. 
// ********************************************************************************  

void AutoDetectProgress(void)
{
	StandardDetect();
	UpdateStandard();
	InputChannelSelectAL242(source); 
	if ( (cNewStandard==STD_NTSC_443)&&(source==1) )
		SetAL242Reg(0x07,0x2A);    //  20h-->2ah  kim0921   Y/C DELAY for S-video with NTSC4.43
	if ( (cNewStandard==STD_PAL_I)&&(source==1) )
		SetAL242Reg(0x03,0x05);     // 03h-->05h  kim0927  change filter for  S-video with PAL-I(reduce ripple)
}



// ********************************************************************************
// Function: AL242Initialize;                  
// Description:    Initialize AL242/240 video decoder to default setting i.e. 
//                 1. NTSC standard
//                 3. Run auto video standard detection mode 
// Input parameters:   None.
// Output parameters:  None. 
// Return values:      None. 
// ********************************************************************************

void AL242Initialize(void)
{
       SetAL242Reg(0xfe,0xff);
	SetAL242Reg(0xb1,0xB2);	     //
	SetAL242Reg(0xb2,0x02);       // output video timing setup   Kim_050808
	SetAL242Reg(0xb3,0x06);        //  same up itself
	SetAL242Reg(0xb4,0x05);       // RTSO2 output 

	SetAL242Reg(0xd5,0x05);
	SetAL242Reg(0xd4,0x3c);
	 SetAL242Reg(0x41,0x01);
	SetAL242Reg(0x51,0x01);
	SetAL242Reg(0x40,0x05);
	SetAL242Reg(0x07,0x00);
	 SetAL242Reg(0x83,0xdf);
	SetAL242Reg(0x02,0x4f);
	 SetAL242Reg(0x1c,0x2b);
	 
	SetAL242Reg(0x1d,0x33);
	 SetAL242Reg(0x1e,0x33);
	SetAL242Reg(0x1f,0x33);
	SetAL242Reg(0xba,0x18);
	
	 SetAL242Reg(0xbb,0x4c);
	SetAL242Reg(0xbc,0xcc);
	
	 SetAL242Reg(0x2d,0x50);
	 
	SetAL242Reg(0x0a,0xb8);
	
//	InitReg(szAL242_INIT);
	InitReg(szNTSC_M);
//	SetAL242Reg(0x08,0xc0);  
//	SetAL242Reg(0x09,0x30);  
//	SetAL242Reg(0x0a,0x80);  
//	SetAL242Reg(0x0b,0x80);  

}

// ********************************************************************************
// Function: dcMoreBrightness
// Description:   AL242 increase brightness.  
// Output parameters:  None. 
// Return values:      None. 
// ********************************************************************************


void dcMoreBrightness()                              
{                                                    
                                                     
    unsigned char value;                                      
                                                     
    value = GetAL242Reg(0x09);     
    if (value <= 0xf7)                                                           
    SetAL242Reg(0x09, value+8);
   else
    SetAL242Reg(0x09, 0xff);

}                                                    
                                                     
// ********************************************************************************
// Function: dcLessBrightness
// Description:   AL242 decrease brightness.  
// Output parameters:  None. 
// Return values:      None. 
// ********************************************************************************
                                  
void dcLessBrightness()                              
{                                                    
                                                     
    unsigned char value;                                      
                                                     
    value = GetAL242Reg(0x09);                          
    if (value >= 0x08)                                                                
    SetAL242Reg(0x09, value-8);
    else 
    SetAL242Reg(0x09, 0x00);
}                                                    
                                                     
// ********************************************************************************
// Function: dcMoreContrast
// Description:   AL242 increase contrast.  
// Output parameters:  None. 
// Return values:      None. 
// ********************************************************************************
                            
void dcMoreContrast()                                
{                                                    
    unsigned char value;                                      
                                                     
    value = GetAL242Reg(0x08);                          
    if (value <= 0xf7)                                                           
    SetAL242Reg(0x08, value+8);
   else
    SetAL242Reg(0x08, 0xff);
}
                                                                                                                  
                                                     
// ********************************************************************************
// Function: dcLessContrast
// Description:   AL242 decrease contrast.  
// Output parameters:  None. 
// Return values:      None. 
// ********************************************************************************
                                  
void dcLessContrast()                                
{                                                                                                
    unsigned char value;                                      
                                                     
    value = GetAL242Reg(0x08);                          
    if (value >= 0x08)                                                                
    SetAL242Reg(0x08, value-8);
    else 
    SetAL242Reg(0x08, 0x00);                                                                                                                
}                                                    

// ********************************************************************************
// Function: dcMoreSaturation
// Description:   AL242 increase saturation.  
// Output parameters:  None. 
// Return values:      None. 
// ********************************************************************************
                               
void dcMoreSaturation()                              
{                                                                                               
    unsigned char value;                                      
                                                     
    value = GetAL242Reg(0x0a);                          
    if (value <= 0xf7)                                                       
    SetAL242Reg(0x0a, value+8);
   else
    SetAL242Reg(0x0a, 0xff);
}                                                        
                                                     
// ********************************************************************************
// Function: dcLessSaturation
// Description:   AL242 decrease saturation.  
// Output parameters:  None. 
// Return values:      None. 
// ********************************************************************************
                                   
void dcLessSaturation()                              
{                                                    
                                                     
    unsigned char value;                                      
                                                     
    value = GetAL242Reg(0x0a);                          
    if (value >= 0x08)                                                                
    SetAL242Reg(0x0a, value-8); 
    else 
    SetAL242Reg(0x0a, 0x00);                     
}                                                    
                                                     
                                                     
// ********************************************************************************
// Function: dcMoreHue
// Description:   AL242 increase hue.  
// Output parameters:  None. 
// Return values:      None. 
// ********************************************************************************
                                  
void dcMoreHue()                                     
{                                                    
                                                     
    unsigned char value;                                      
                                                     
    value = GetAL242Reg(0x0b);                          
    if (value <= 0xf7)                                                          
    SetAL242Reg(0x0b, value+8);
   else
    SetAL242Reg(0x0b, 0xff);
}      

// ********************************************************************************
// Function: dcLessHue
// Description:   AL242 decrease hue.  
// Output parameters:  None. 
// Return values:      None. 
// ********************************************************************************
                                 
void dcLessHue()                                     
{                                                    
                                                     
    unsigned char value;                                      
                                                     
    value = GetAL242Reg(0x0b);                          
    if (value >= 0x08)                                                                
    SetAL242Reg(0x0b, value-8);
    else 
    SetAL242Reg(0x0b, 0x00);                                         
}                                                    

// ********************************************************************************
// Function: dcGetBrightness
// Description:   get current AL242 brightness value.  
// Output parameters:  None. 
// Return values:      current AL242 brightness value.
// ********************************************************************************

unsigned char dcGetBrightness()                               
{                                                    
                                                     
    return(GetAL242Reg(0x09));                          
                                                     
                                                     
}                  

// ********************************************************************************
// Function: dcGetContrast
// Description:   get current AL242 Contrast value.  
// Output parameters:  None. 
// Return values:      current AL242 Contrast value.
// ********************************************************************************
                         
unsigned char dcGetContrast()                                 
{                                                    
                                                     
    return(GetAL242Reg(0x08));                      
                                                     
}                                                    
                    
// ********************************************************************************
// Function: dcGetSaturation
// Description:   get current AL242 Saturation value.  
// Output parameters:  None. 
// Return values:      current AL242 Saturation value.
// ********************************************************************************

unsigned char dcGetSaturation()                               
{                                                    
                                                     
    return(GetAL242Reg(0x0a));                      
                                                     
                                                     
                                                     
}                                                    
                                                     
// ********************************************************************************
// Function: dcGetHue
// Description:   get current AL242 Hue value.  
// Output parameters:  None. 
// Return values:      current AL242 Hue value.
// ********************************************************************************

unsigned char dcGetHue()                                      
{                                                    
                                                     
    return(GetAL242Reg(0x0b));                      
                                                     
                                                     
}                                                    
                                                     
                                                 
                                                     

