#include <reg51.h>
#include "AL242.h"
#include "AL251.h"

 
// ********************************************************************************
// Function: scl_toggle.                  
// Description:  Sending a high - low translate waveform in I2C bus scl pin. 
// Input parameters:   None.
// Output parameters:  None. 
// Return values:      None. 
// ********************************************************************************  

void al242_scl_toggle(void)
{
	set_scl(HIGH);
	set_scl(HIGH);
	set_scl(LOW);
	set_scl(LOW);
	set_scl(HIGH);
	set_scl(HIGH);
	set_scl(LOW);
	set_scl(LOW);
	set_scl(HIGH);
	set_scl(HIGH);
}

// ********************************************************************************
// Function: i2c_start;                  
// Description:  Sending a start waveform to I2C bus. 
// Input parameters:   None.
// Output parameters:  None. 
// Return values:      None. 
// ******************************************************************************** 
void al242_i2c_start(void)
{
	set_sda(HIGH);
	set_scl(HIGH);
	set_sda(HIGH);
	set_sda(LOW);
	set_scl(LOW);
	set_scl(LOW);

}

// ********************************************************************************
// Function: i2c_stop;                  
// Description:  Sending a stop waveform to I2C bus.           				 		    
// Input parameters:   None.
// Output parameters:  None. 
// Return values:      None. 
// ******************************************************************************** 

void al242_i2c_stop(void)
{
/*	idata unsigned char _IDCodeOfI2C;
	unsigned long _timeOut=0;*/
	set_sda(LOW);
	set_sda(LOW);
	set_scl(HIGH);
	set_scl(HIGH);
/*	if( (_IDCodeOfI2C==0xb8) || (_IDCodeOfI2C==0xba) )
	{
	   while( (i2c_scl==0) && (_timeOut<0x4f000) )
		 {
			alDelay(1);
		       _timeOut++;	
		 }
	}*/
	set_sda(LOW);
	set_sda(LOW);
	set_scl(HIGH);
	set_sda(HIGH);
}
 
// ********************************************************************************
// Function: i2c_write_byte;                  
// Description:  Transmit 1 byte of data to I2C slave end. 
// Input parameters:   cDat:
//                     Data been transmited. 
// Output parameters:  None. 
// Return values:      None. 
// ********************************************************************************

void al242_i2c_write_byte(unsigned char cDat)
{
	unsigned char i;
	// shift 8 bits data 
	for (i = 0; i < 8; i++) 
	{
		if (cDat & 0x80)
		{
			set_sda(HIGH);
		}
		else
		{
			set_sda(LOW);
		}

		set_scl(HIGH);
		set_scl(HIGH);
		set_scl(HIGH);
		set_scl(HIGH);

		set_scl(LOW);
		set_scl(LOW);
		set_scl(LOW);
		set_scl(LOW);

		cDat <<= 1;
	}		
	// ACK
	set_sda_input;
	set_sda_input;
	set_sda_input;

	set_scl(HIGH);
	set_scl(HIGH);

	set_scl(LOW);
	set_scl(LOW);
}

// ********************************************************************************
// Function:  i2c_read_byte;                  
// Description:  Receive 1 byte of data from I2C slave end.            				 		    
// Input parameters:   pcDat:
//                     pointer to data been received. 
// Output parameters:  None. 
// Return values:      None. 
// ******************************************************************************** 

unsigned char al242_i2c_read_byte(void)
{
	unsigned char value,i;
	set_sda_input;
	value = 0;
	// shift 8 bits data 

	set_scl(LOW);
	set_scl(LOW);

	for (i = 0; i < 8; i++) 
	{

		set_scl(HIGH);
		set_scl(HIGH);
		set_scl(HIGH);
		set_scl(HIGH);

		value <<= 1;
		if (i2c_sda_al242)
		{
			value++;
		}
		set_scl(LOW);
		set_scl(LOW);
		set_scl(LOW);
		set_scl(LOW);
	}
	
	// ACK
	set_sda(HIGH);
	set_scl(HIGH);
	set_sda(HIGH);
	set_scl(HIGH);

	set_scl(LOW);
	set_scl(LOW);
	set_scl(LOW);
	set_scl(LOW);
    return (value);
}

// ********************************************************************************
// Function:   i2c_write;                  
// Description: Transmit 1 byte of data to slave end with assigned ID and sub address. 
// Input parameters:   cChip_addr:   I2C slave ID of destination chip. 
//                     cIndex:  sub address of destination chip. 
//					   cDat:  Data been transmited.
// Output parameters:  None. 
// Return values:      TRUE=1. 
// ********************************************************************************  

unsigned char al242_i2c_write(unsigned char cChip_addr, unsigned char cIndex, unsigned char cDat)
{
	// keep scl output high
	al242_scl_toggle();

	// I2C start
	al242_i2c_start();

	// Chip address
	al242_i2c_write_byte(cChip_addr & 0xfe);

	
	// Index
	al242_i2c_write_byte(cIndex);

	// Data
	al242_i2c_write_byte(cDat);

	// I2C stop
	al242_i2c_stop();
//	}
	return TRUE;
}

// ********************************************************************************
// Function:   i2c_read;                  
// Description: Receive a chunk of data from slave end with assigned ID and sub address. 
// Input parameters:   cChip_addr:   I2C slave ID of destination chip. 
//                     cIndex:  sub address of destination chip. 
//					   pcDat:  pointer of data been received. 
// Output parameters:  None. 
// Return values:      TRUE=1. 
// ******************************************************************************** 

unsigned char al242_i2c_read(unsigned char cChip_addr, unsigned char cIndex )
{

	unsigned char value;
	// keep scl output high
	al242_scl_toggle();

	// I2C start
	al242_i2c_start();

	// Chip address
	al242_i2c_write_byte(cChip_addr & 0xfe);

	// Index
	al242_i2c_write_byte(cIndex);

	// I2C stop
	al242_i2c_stop();

	// I2C start
	al242_i2c_start();


	// Chip address
	al242_i2c_write_byte(cChip_addr | 0x01);
	
	// Index
	value = al242_i2c_read_byte();

	// I2C stop
	al242_i2c_stop();
 
 	return (value);

}


// ********************************************************************************
// Function:  GetAL242Reg;                  
// Description: Receive 1 byte of data from slave end with assigned ID and sub address.            				 		    
// Input parameters:    _SlaveID:   I2C slave ID of destination chip. 
//                      _SubAddr:  sub address of destination chip. 					   
// Output parameters:  _dx: byte of data been received from slave end. 
// Return values:      _dx: byte of data been received from slave end. 
// ********************************************************************************

unsigned char GetAL242Reg(unsigned char _SubAddr)
{
	unsigned char value;
	value = al242_i2c_read(AL242_slaveID,_SubAddr);
	 return(value);
}

// ********************************************************************************
// Function:  SetAL242Reg;                  
// Description: Transmite 1 byte of data to slave end with assigned ID and sub address.            				 		    
// Input parameters:    _SlaveID:   I2C slave ID of destination chip. 
//                               _SubAddr:  sub address of destination chip. 
//				       _dx:       data been transmited.
// Output parameters:   None. 
// Return values:       None. 
// ******************************************************************************** 

void SetAL242Reg(unsigned char _SubAddr,unsigned char _dx)
{
	al242_i2c_write(AL242_slaveID,_SubAddr,_dx); 
} 






