/*----------------------------------------------------------------------------
 *  Copyright 1996,1997     Averlogic Technologies Inc.
 *----------------------------------------------------------------------------*/

#include <reg51.h>
#include "AL251.h"
#define MAX_RETRY   3


/*
 * Functions provided by Master chip producer
 */ 
/*void i2cEnable();*/
/*void i2cDisable();*/
                                             
/*
 * Local functions                                             
 */
unsigned char i2cLocate(unsigned char slave_addr, unsigned char  sub_addr);
void i2cStart(void);
void i2cStop(void);
void i2cTransmit(unsigned char /*send_data*/);
unsigned char i2cWaitAck(void);
void i2cNotAck(void);
void i2cGiveAck();
unsigned char i2cReceive(void);
void i2cSetSCL1(void);
void i2cSetSCL0(void);
void i2cSetSDA1(void);
void i2cSetSDA0(void);
unsigned char i2cGetSDA(void);
void i2cReleaseSDA(void);


/*
 *Exported functions
 */
unsigned char i2cWrite(unsigned char slave_addr, unsigned char  sub_addr, unsigned char value)
{
    if (i2cLocate(slave_addr, sub_addr))
    {
        /*value */
        i2cTransmit(value);
        if (i2cWaitAck() == 0)
        {    
            i2cStop();   
            return(0);
        }        
              
        /*Stop I2C            */
        i2cStop();   
             
        return(1);     
    }
    return(0);
}

unsigned char i2cRead(unsigned char slave_addr, unsigned char  sub_addr)
{      
    unsigned char value;
    
    if (i2cLocate(slave_addr,  sub_addr))
    {
        //Read data byte pointed                            
        //I2C Start                 
        i2cStart();

        i2cTransmit((unsigned char)(slave_addr | 0x0001));  /*read*/
        if (!i2cWaitAck())
        {    
            i2cStop();
            return(0xff);
        }        
        value = i2cReceive();
        i2cNotAck();
               
        i2cStop();
        
        /*Succeed!             */
        return(value);     
    }
    return(0xff);
}

unsigned char i2cLocate(unsigned char slave_addr, unsigned char  sub_addr)
{
    unsigned char i;
                     
    for (i=0; i<MAX_RETRY; i++)
    {    
        /*Start I2C */             
        i2cStart();
    
        /*Slave address    */
        i2cTransmit((unsigned char)slave_addr);
        if (i2cWaitAck() == 0)
        {    
            i2cStop();   
            continue;
        }        
                       
        /*Subaddress         */          
        i2cTransmit((unsigned char)sub_addr);
        if (i2cWaitAck() == 0)
        {    
            i2cStop();   
            continue;
        }        
                    
        return(1);     
    }

    /* Fail */    
    return(0);     
} 

/*
 *Local functions
 */
void i2cStart(void)
{
    i2cSetSCL0();
    i2cSetSDA1();
    i2cSetSCL1();
    i2cSetSDA0();
    i2cSetSCL1();
}





void i2cStop(void)
{
    i2cSetSCL0();
    i2cSetSDA0();
    i2cSetSCL1();
    i2cSetSDA1();
    i2cSetSCL1();
    /*i2cSetSCL0();*/
}

void i2cTransmit(unsigned char send_data)
{
    unsigned char i;

    /*Set value bit by bit...MSB first*/
    for (i=0; i<8; i++)
    {                                                        
        i2cSetSCL0();
        if (send_data & 0x80)
            i2cSetSDA1();
        else
            i2cSetSDA0();
        i2cSetSCL1();
        
        send_data <<= 1;
    }
    i2cSetSCL0();
    
    i2cReleaseSDA();
}
                         
#define MAX_WAIT_COUNT  254                         
unsigned char i2cWaitAck(void)
{
    unsigned char   count = 0L;
        
    /*transmit state, ack set by receiver*/
    i2cSetSCL0();

    while (i2cGetSDA() && count++ <= MAX_WAIT_COUNT);
    
    if (i2cGetSDA())
        return(0);

    i2cSetSDA0();
    i2cSetSCL1();
    i2cSetSCL0();
    return(1);
}

void i2cNotAck(void)
{
    /*receive state, ack set by master*/
    i2cSetSCL0();
    i2cSetSDA1();
    i2cSetSCL1();
    i2cSetSCL0();
}

/*void i2cGiveAck(void)
{
    //receive state, ack set by master
    i2cSetSCL0();
    i2cSetSDA0();
    i2cSetSCL1();   //SCL to CLOCK_HIGH
    i2cSetSCL0();
}*/

unsigned char i2cReceive(void)
{
    /*Read ack.*/  
    unsigned char     i;
    unsigned char    value;                                                     
    
    i2cReleaseSDA();

    /*Set value bit by bit...MSB first*/
    value = 0;
    for (i=0; i<8; i++)
    {                                                        
        value <<= 1;
        i2cSetSCL0();
        value |= i2cGetSDA();               
        i2cSetSCL1();
    }
    i2cSetSCL0();

    return(value);    
}


void i2cSetSCL1(void)
{
    i2c_scl = 1;
    /*_scl = 1;*/
}

void i2cSetSCL0(void)
{
    i2c_scl = 0;
    /*_scl = 0;*/
}

void i2cSetSDA1(void)
{
       i2c_sda = 1;
}

void i2cSetSDA0(void)
{
        i2c_sda = 0;

}

unsigned char i2cGetSDA(void)
{
        if (i2c_sda == 0)
            return(0);
    return(1);    
}    

void i2cReleaseSDA(void)
{
       i2c_sda = 1;   /*pull high*/ 
}    






