#define SCL       P1_0    // I2C Clock
#define SDA       P1_1    // I2C Data
#define READ_F      0x01    // Alias for read command bit
#define WRITE_F     0x00    // Alias for write command bit
#define EEPROM_ID	0x58

void i2c_delay(void);                         // Wait for 5.425s
void i2c_start(void);                         // Geneate I2C START condition
void i2c_stop(void);                          // Geneate I2C STOP condition
void i2c_ack(void);                           // Geneate I2C MASTER ACK
void i2c_nack(void);                          // Geneate I2C MASTER NACK
unsigned char i2c_write(unsigned char value); // Write a character via I2C
unsigned char i2c_check(void);                // Check an ACK from SLAVE
unsigned char i2c_read(void);                 // Read a character via I2C

void errormsg(unsigned char ascii);


//----------------------------------------------------------------------
void i2c_delay(void)
{
	unsigned char d;

	for(d=2;d>0;d--);
  	

}
//----------------------------------------------------------------------
void i2c_start(void)
{
  

  // Force defined state of SCL and SDA
  SCL = 1;       // Release SCL
  SDA = 1;       // Release SDA
  i2c_delay();   // Force 5s-delay

  // Generate START condition
  SDA = 0;
  i2c_delay();   // Force 5s-delay
  SCL = 0;

  
  return;
}
//----------------------------------------------------------------------
void i2c_stop(void)
{
  
  // Generate STOP condition
  SDA = 0;
  i2c_delay();   // Force 5s-delay
  SCL = 1;       // Release SCL
  i2c_delay();   // Force 5s-delay
  SDA = 1;       // Release SDA
  
  return;
}
//----------------------------------------------------------------------
void i2c_ack(void)
{
  //EA = 0;            // Disable all interrupts for a accurate timing

  // Generate ACK
  SDA = 0;
  SCL = 1;           // Release SCL
  while(SCL == 0);   // Synchronize clock
  
  i2c_delay();       // Force 5s-delay
  SCL = 0;           // Force a clock cycle
  
  //EA = 1;            // Enable all interrupts
  
  return;
}
//----------------------------------------------------------------------
void i2c_nack(void)
{
  //EA = 0;            // Disable all interrupts for a accurate timing

  // Generate NACK
  SDA = 1;           // Release SDA
  SCL = 1;           // Release SCL
  while(SCL == 0);   // Synchronize clock
  
  i2c_delay();       // Force 5s-delay
  SCL = 0;           // Force a clock cycle

  //EA = 1;            // Enable all interrupts
  
  return;
}
//----------------------------------------------------------------------
unsigned char i2c_check(void)
{
  //EA = 0;            // Disable all interrupts for a accurate timing

  SDA = 1;           // Release SDA
  SCL = 1;           // Release SCL
  while(SCL == 0);   // Synchronize clock
  i2c_delay();       // Force 5s-delay
  if (SDA)           // SDA is high
  {
    SCL = 0;          // Force a clock cycle
    return(1);        // No acknowledge from Slave
  }
  SCL = 0;           // Force a clock cycle

  //EA = 1;            // Enable all interrupts

  return(0);         // Acknowledge from Slave
}
//----------------------------------------------------------------------
unsigned char i2c_write(unsigned char value)
{
  unsigned char counter;  // Bitcounter

  //EA = 0;                           // Disable all interrupts for a accurate timing

  for ( counter = 8; counter > 0; counter-- )
  {
    SDA = (bit)(((value)&0x80) /*>> 7*/); // Set SDA to value of MSB
    value <<= 1;                    // Prepare next bit for transmission
	SCL = 1;                        // Release SCL
    while(SCL == 0);                // Synchronize clock
    i2c_delay();                    // Force 5s-delay
    SCL = 0;                        // Force a clock cycle
  }

  //EA = 1;                           // Enable all interrupts

  // Generate a 9. clock cycle and check ACK from SLAVE
  // Return the result
  return(i2c_check());
}
//----------------------------------------------------------------------
unsigned char i2c_read(void)
{
  unsigned char result  = 0;  // Returnvalue with read I2C byte
  unsigned char counter = 0;  // Bitcounter

  //EA = 0;              // Disable all interrupts for a accurate timing

  for ( counter = 8; counter > 0; counter-- )
  {
    SDA = 1;           // Release SDA
    SCL = 1;           // Release SCL
    while(SCL == 0);   // Synchronize clock
    i2c_delay();       // Force 5s-delay

    result <<= 1;      // Shift left the result

    if (SDA) {result |= 0x01;}  // Set actual SDA state to LSB

    SCL = 0;           // Force a clock cycle
    i2c_delay();       // Force 5s-delay
  }

  //EA = 1;              // Enable all interrupts

  // ACK or NACK from MASTER must be generated outside this routine

  return(result);
}
//----------------------------------------------------------------------

void EEPROM_write(unsigned char adress, unsigned char content)
{
 unsigned int i=0;

  i2c_start();   // Generate START condition

  // Send SLAVE adress with write request
  if ( i2c_write(EEPROM_ID|WRITE_F) )
  {
    errormsg('1');
    return;
  }

//i2c_ack();

  // Send memory adress
  if ( i2c_write(adress) )
  {
    errormsg('2');
    return;
  }

//i2c_ack();

  // Send content to memory adress
  if ( i2c_write(content) )
  {
    errormsg('3');
    return;
  }

//i2c_ack();

  i2c_stop();    // Generate STOP condition

  //delay(100);    // Secure wait for the EEPROMs write cycle
  for (i=0;i<20;i++) i=i;

  return;
}

unsigned char EEPROM_read(unsigned char adress)
{
  unsigned char result = 0;

  i2c_start();   // Generate START condition

  // Send SLAVE adress with dummy write request
  if ( i2c_write(EEPROM_ID|WRITE_F) )
  {
    errormsg('4');
    return(0);
  }

  // Send memory adress
  if ( i2c_write(adress) )
  {
    errormsg('5');
    return(0);
  }

  i2c_start();   // Generate START condition

  // Send SLAVE adress with read request
  if ( i2c_write(EEPROM_ID|READ_F) )
  {
    errormsg('6');
    return(0);
  }

  result = i2c_read();   // Read memory content
  // Dont perform a MASTER ACK !
  i2c_stop();            // Generate STOP condition

  return(result);
}

//----------------------------------------------------------------------
void errormsg(unsigned char ascii)
{
 // while(1){}
  return;
}

