#include "al250.h"
#include <math.h>

#ifndef TYPEDEF_BYTE
#define TYPEDEF_BYTE
typedef unsigned char BYTE;
#endif
#ifndef TYPEDEF_USHORT
#define TYPEDEF_USHORT
typedef unsigned short USHORT;
#endif

#define RED_BOOT    56
#define GREEN_BOOT  28
#define BLUE_BOOT   36

#define OSD_END     4

#define OP_COPY     0
#define OP_AND      1
#define OP_OR       2
#define OP_XOR      3

/*
 * OSD H/V position
 */               
#ifdef CCIR 
#define OSD_ICON_HSTART (3<<3)
#define OSD_BITMAP_HSTART (4<<3)
#define OSD_ICON_CENTER_HSTART (6<<3)
#define OSD_BITMAP_CENTER_HSTART (3<<3)
#else
#define OSD_ICON_HSTART (3<<3)
#define OSD_BITMAP_HSTART (4<<3)
#define OSD_ICON_CENTER_HSTART (7<<3)
#define OSD_BITMAP_CENTER_HSTART (4<<3)
#endif
#define OSD_VSTART (4<<4)

/* main.c */
unsigned char alGetRegister(unsigned char index);
void alSetRegister(unsigned char index, unsigned char value);

/* local */ 
void dblBootLut(BYTE R, BYTE G, BYTE B);
void dblHome(void);
#if 0
void dlSinBootLut(float fboot);
#else
void dlSinBootLut(BYTE boot);
#endif
void dblSetBuiltinOSD(void/*BYTE d0, BYTE d1, BYTE d2, BYTE d3*/);
void dblInitBuiltinOSD(void);
void dblSetOverlayColor(BYTE i, BYTE r0, BYTE g0, BYTE b0);
void dblSetBorderColor(BYTE r, BYTE g, BYTE b);

BYTE video_system = 0;
BYTE lut = 0;
BYTE bootlut;
BYTE osd_state = 0; /* must start from 0 */
BYTE ovl_state = 0; /* must start from 0 */
BYTE crop_state = 0; /* must start from 0 */
BYTE osd_index;
BYTE d0, d1, d2, d3;
BYTE value;
void dblInitialize(void)
{
    dblInitBuiltinOSD();
    dblHome();                              
}

void dblHome(void)
{
    if (bootlut != 1)   /* default already done */
    {
        bootlut = 0;
        OnBootLut();
    }

    /* no external overlay */
    ovl_state = 0;

    /* no internal OSD */
    osd_state = 0;
    OnBuiltinOSD();

    
    lut = 0;
    OnLut();

    crop_state = 0;    
    /* default border color */
    dblSetBorderColor(0, 0, 0);

    dblSetSystem(video_system, 0);
}

/* check if auto detection */
#define SYS_CHECK   30
BYTE dblAutoDetect(void)
{
    for (value=0; value<SYS_CHECK; value++)
    {
        if ((alGetRegister(0x09) & 0x01) == 0x00)
            return(0);
    }                
    return(1);
}

void dblSetSystem(BYTE system, char crop)
{                                                           
    /* 
     *you can comment all the codes in this function 
     * if you want use hardware defaults
     */

    video_system = system;
#if 1     /* for demo, you must turn on S/W timing */
#if defined(CCIR)
    if (system == 0)
    {
        alSetRegister(0x20, 0x09+crop);      /* HDE start = 72 */
        alSetRegister(0x21, 0x5e-crop);      /* HDE end = 752 */
        alSetRegister(0x22, 0x63);      /* HSYNC start = 792 */
        alSetRegister(0x23, 0x6b);      /* HSYNC end = 856 */
        alSetRegister(0x24, 0x6b);      /* HTOTAL = 858 */ 
        alSetRegister(0x29, 0x01);
        alSetRegister(0x25, 0x0c+crop);      /* VDE start = 48 */
        alSetRegister(0x26, 0x7d-crop);      /* VDE end = 500 */
        alSetRegister(0x27, 0x01);      /* VSYNC start = 4 */
        alSetRegister(0x28, 0x02);      /* VSYNC end = 8 */
        /* border */
        alSetRegister(0x2c, 0x09-2);      /* HBLANK end = 72 - 16 */
        alSetRegister(0x2b, 0x5e+3);      /* HBLANK start = 752 + 16  */
        alSetRegister(0x2e, 0x0c-3);      /* VBLANK end = 48 - 12 */
        alSetRegister(0x2d, 0x7d+3);      /* VBLANK start = 500 + 12  */
    }
    else
    {
        alSetRegister(0x20, 0x0a+crop);      /* HDE start = 80 */
        alSetRegister(0x21, 0x5f-crop);      /* HDE end = 760 */
        alSetRegister(0x22, 0x65);      /* HSYNC start = 808 */
        alSetRegister(0x23, 0x02);      /* HSYNC end = 16 */
        alSetRegister(0x24, 0x6c);      /* HTOTAL = 864 */ 
        alSetRegister(0x29, 0x00);
        alSetRegister(0x25, 0x0f+crop);      /* VDE start = 60 */
        alSetRegister(0x26, 0x97-crop);      /* VDE end = 604 */
        alSetRegister(0x27, 0x01);      /* VSYNC start = 4 */
        alSetRegister(0x28, 0x02);      /* VSYNC end = 8 */
        /* Border */
        alSetRegister(0x2c, 0x0a-2);      /* HBLANK end = 80 - 16 */
        alSetRegister(0x2b, 0x5f+3);      /* HBLANK start = 760 + 16  */
        alSetRegister(0x2e, 0x0f-3);      /* VBLANK end = 60 - 12 */
        alSetRegister(0x2d, 0x97+3);      /* VBLANK start = 604 + 12 */
    }
#else /* SQUARE */
    if (system == 0)
    {
        alSetRegister(0x20, 0x0f+crop);      /* HDE start = 120 */
        alSetRegister(0x21, 0x5c-crop);      /* HDE end = 736 */
        alSetRegister(0x22, 0x61);      /* HSYNC start = 776 */
        alSetRegister(0x23, 0x07);      /* HSYNC end = 56 */
        alSetRegister(0x24, 0x61);      /* HTOTAL = 780 */ 
        alSetRegister(0x29, 0x02);
        alSetRegister(0x25, 0x0c+crop);      /* VDE start = 48 */
        alSetRegister(0x26, 0x7d-crop);      /* VDE end = 500 */
        alSetRegister(0x27, 0x01);      /* VSYNC start = 4 */
        alSetRegister(0x28, 0x02);      /* VSYNC end = 8 */
        /* Border */
        alSetRegister(0x2c, 0x0f-2);      /* HBLANK end = 120 - 16 */
        alSetRegister(0x2b, 0x5c+3);      /* HBLANK start = 736 + 16 */
        alSetRegister(0x2e, 0x0c-3);      /* VBLANK end = 48 - 12 */
        alSetRegister(0x2d, 0x7d+3);      /* VBLANK start = 500 + 12 */
    }
    else
    {
        alSetRegister(0x20, 0x14+crop);      /* HDE start = 160 */
        alSetRegister(0x21, 0x70-crop);      /* HDE end = 896 */
        alSetRegister(0x22, 0x00);      /* HSYNC start = 0 */
        alSetRegister(0x23, 0x09);      /* HSYNC end = 72 */
        alSetRegister(0x24, 0x76);      /* HTOTAL = 944 */ 
        alSetRegister(0x29, 0x00);
        alSetRegister(0x25, 0x0f);      /* VDE start = 60 */
        alSetRegister(0x26, 0x97);      /* VDE end = 604 */
        alSetRegister(0x27, 0x01);      /* VSYNC start = 4 */
        alSetRegister(0x28, 0x02);      /* VSYNC end = 8 */
        /* Border */
        alSetRegister(0x2c, 0x14-2);      /* HBLANK end = 160 - 16 */
        alSetRegister(0x2b, 0x70+3);      /* HBLANK start = 896 + 16 */
        alSetRegister(0x2e, 0x0f-3);      /* VBLANK end = 60 - 12 */
        alSetRegister(0x2d, 0x97+3);      /* VBLANK start = 604 + 12 */
    }
#endif    
    alSetRegister(0x08, 0x08);  /* enable software timing */    
#endif
}

/*
 *  Button
 */     
void OnExternalOverlay(void)
{
    /* disable internal OSD */
    alSetRegister(0x10, alGetRegister(0x10) & ~0xc0);  /* icon & bitmap off */
    osd_state = 0;
    ovl_state++;
    if (ovl_state == 1)
    {
        /* Negative effect */
        alSetRegister(0x2a, 0x10);
        dblSetOverlayColor(0, 255, 255, 255);
        alSetRegister(0x14, OP_XOR);
    }
    else if (ovl_state >= 2 && ovl_state <= 4)
    {
        /* R, G, B effect */
        /*alSetRegister(0x2a, 0x10);*/
        dblSetOverlayColor(0, 
                           ovl_state == 2 ? 255 : 0, 
                           ovl_state == 3 ? 255 : 0, 
                           ovl_state == 4 ? 255 : 0);
        alSetRegister(0x14, OP_AND);
    }
    else if (ovl_state == 5)
    {
        /* solization effect */
        /*alSetRegister(0x2a, 0x10);*/
        dblSetOverlayColor(0, 0xe0, 0xe0, 0xe0);
        /*alSetRegister(0x14, OP_AND);*/
    }
    else
    {
        /* turn off effect */
        alSetRegister(0x2a, 0x00);  /* external overlay */
        ovl_state = 0;
    }
}
                   
void OnBuiltinOSD(void)
{
    /* clear overlay screen */
    alSetRegister(0x2a, 0x00);  /* external overlay */
                                          
    /* overlay color setup */
    dblSetOverlayColor(0, 0, 0, 255);
    dblSetOverlayColor(3, 255, 0, 0);
    dblSetOverlayColor(6, 255, 255, 0);
     
    value = alGetRegister(0x10) & ~0xc0;
    if (osd_state == 1)
    {
        /*show Icon & bitmap, mesh off*/
        alSetRegister(0x10, value | 0xc0);
        alSetRegister(0x1e, OSD_ICON_HSTART);
        alSetRegister(0x1f, OSD_BITMAP_HSTART);
        alSetRegister(0x2f, OSD_VSTART);    /* mesh off*/
        osd_state = 2;
    }                                   
    else if (osd_state == 2)
    {
        /*show Icon & bitmap, mesh on*/
        /* state must start from 0 */
        /*alSetRegister(0x10, value | 0xc0);*/
        /*alSetRegister(0x1e, OSD_ICON_HSTART);*/
        /*alSetRegister(0x1f, OSD_BITMAP_HSTART);*/
        alSetRegister(0x2f, OSD_VSTART | 1);    /* grey mesh */
        osd_state = 0;
    } 
    else 
    {
        /* disable bitmap & icon */
        alSetRegister(0x10, value);  /* icon & bitmap off */
        osd_state = 1;
    }
        
    return;        
}
      
void dblSetOverlayColor(BYTE i, BYTE r0, BYTE g0, BYTE b0)
{
    alSetRegister(0x15+i, r0);
    alSetRegister(0x16+i, g0);
    alSetRegister(0x17+i, b0);
}
    
void dblSetBorderColor(BYTE r, BYTE g, BYTE b)
{
    alSetRegister(0x0c, r);
    alSetRegister(0x0d, g);
    alSetRegister(0x0e, b);
}
           
void dblInitBuiltinOSD(void)
{           
    /* enable OSD I/O */
    alSetRegister(0x10, alGetRegister(0x10) | 0x03);
    
    /* OSD bitmap */
    osd_index = 0;
      d0=0x00;     d1=0x00;     d2=0x00;     d3=0x00;   dblSetBuiltinOSD(/*0x00, 0x00, 0x00, 0x00*/); 
    /*d0=0x00;*/   d1=0x0a;     d2=0xa0;   /*d3=0x00;*/ dblSetBuiltinOSD(/*0x00, 0x0a, 0xa0, 0x00*/); 
    /*d0=0x00;*/   d1=0x20;     d2=0x08;   /*d3=0x00;*/ dblSetBuiltinOSD(/*0x00, 0x20, 0x08, 0x00*/); 
    /*d0=0x00;*/   d1=0x80;     d2=0x02;   /*d3=0x00;*/ dblSetBuiltinOSD(/*0x00, 0x80, 0x02, 0x00*/); 
      d0=0x02;     d1=0x00;     d2=0x00;     d3=0x80;   dblSetBuiltinOSD(/*0x02, 0x00, 0x00, 0x80*/); 
    /*d0=0x02;*/ /*d1=0x00;*/ /*d2=0x00;*/ /*d3=0x80;*/ dblSetBuiltinOSD(/*0x02, 0x00, 0x00, 0x80*/); 
    /*d0=0x02;*/ /*d1=0x00;*/ /*d2=0x00;*/ /*d3=0x80;*/ dblSetBuiltinOSD(/*0x02, 0x00, 0x00, 0x80*/); 
      d0=0x08;   /*d1=0x00;*/   d2=0x3f;     d3=0x20;   dblSetBuiltinOSD(/*0x08, 0x00, 0x3f, 0x20*/); 
    /*d0=0x08;*/   d1=0x0f;     d2=0xff;   /*d3=0x20;*/ dblSetBuiltinOSD(/*0x08, 0x0f, 0xff, 0x20*/); 
    /*d0=0x08;*/   d1=0xff;     d2=0xf0;   /*d3=0x20;*/ dblSetBuiltinOSD(/*0x08, 0xff, 0xf0, 0x20*/); 
      d0=0x20;     d1=0xf0;     d2=0x30;     d3=0x08;   dblSetBuiltinOSD(/*0x20, 0xf0, 0x30, 0x08*/); 
    /*d0=0x20;*/   d1=0xff;     d2=0xf0;   /*d3=0x08;*/ dblSetBuiltinOSD(/*0x20, 0xff, 0xf0, 0x08*/); 
    /*d0=0x20;*/   d1=0x0f;     d2=0xff;   /*d3=0x08;*/ dblSetBuiltinOSD(/*0x20, 0x0f, 0xff, 0x08*/); 
    /*d0=0x20;*/   d1=0x00;     d2=0x3f;   /*d3=0x08;*/ dblSetBuiltinOSD(/*0x20, 0x00, 0x3f, 0x08*/); 
    /*d0=0x20;*/ /*d1=0x00;*/   d2=0x00;   /*d3=0x08;*/ dblSetBuiltinOSD(/*0x20, 0x00, 0x00, 0x08*/); 
      d0=0x80;     d1=0xff;     d2=0xff;     d3=0x02;   dblSetBuiltinOSD(/*0x80, 0xff, 0xff, 0x02*/); 
    /*d0=0x80;*/ /*d1=0xff;*/ /*d2=0xff;*/ /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0xff, 0xff, 0x02*/); 
    /*d0=0x80;*/   d1=0x00;     d2=0x03;   /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x00, 0x03, 0x02*/); 
    /*d0=0x80;*/ /*d1=0x00;*/ /*d2=0x03;*/ /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x00, 0x03, 0x02*/); 
    /*d0=0x80;*/ /*d1=0x00;*/ /*d2=0x03;*/ /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x00, 0x03, 0x02*/); 
    /*d0=0x80;*/ /*d1=0x00;*/ /*d2=0x03;*/ /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x00, 0x03, 0x02*/); 
    /*d0=0x80;*/ /*d1=0x00;*/   d2=0x00;   /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x00, 0x00, 0x02*/); 
    /*d0=0x80;*/   d1=0x10;     d2=0x05;   /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x10, 0x05, 0x02*/); 
    /*d0=0x80;*/   d1=0x50;     d2=0x15;   /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x50, 0x15, 0x02*/); 
    /*d0=0x80;*/   d1=0x40;     d2=0x51;   /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x40, 0x51, 0x02*/); 
    /*d0=0x80;*/   d1=0x55;     d2=0x41;   /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x55, 0x41, 0x02*/); 
    /*d0=0x80;*/   d1=0x15;     d2=0x01;   /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x15, 0x01, 0x02*/); 
    /*d0=0x80;*/   d1=0x00;     d2=0x00;   /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x00, 0x00, 0x02*/); 
    /*d0=0x80;*/   d1=0x15;     d2=0x04;   /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x15, 0x04, 0x02*/); 
    /*d0=0x80;*/   d1=0x55;     d2=0x05;   /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x55, 0x05, 0x02*/); 
    /*d0=0x80;*/   d1=0x41;     d2=0x01;   /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x41, 0x01, 0x02*/); 
    /*d0=0x80;*/ /*d1=0x41;*/   d2=0x55;   /*d3=0x02;*/ dblSetBuiltinOSD(/*0x80, 0x41, 0x55, 0x02*/); 
      d0=0x20;     d1=0x40;     d2=0x54;     d3=0x08;   dblSetBuiltinOSD(/*0x20, 0x40, 0x54, 0x08*/); 
    /*d0=0x20;*/   d1=0x00;     d2=0x00;   /*d3=0x08;*/ dblSetBuiltinOSD(/*0x20, 0x00, 0x00, 0x08*/); 
    /*d0=0x20;*/   d1=0x15;     d2=0x54;   /*d3=0x08;*/ dblSetBuiltinOSD(/*0x20, 0x15, 0x54, 0x08*/); 
    /*d0=0x20;*/   d1=0x55;     d2=0x55;   /*d3=0x08;*/ dblSetBuiltinOSD(/*0x20, 0x55, 0x55, 0x08*/); 
    /*d0=0x20;*/   d1=0x40;     d2=0x01;   /*d3=0x08;*/ dblSetBuiltinOSD(/*0x20, 0x40, 0x01, 0x08*/); 
      d0=0x08;     d1=0x55;     d2=0x55;     d3=0x20;   dblSetBuiltinOSD(/*0x08, 0x55, 0x55, 0x20*/); 
    /*d0=0x08;*/   d1=0x15;     d2=0x54;   /*d3=0x20;*/ dblSetBuiltinOSD(/*0x08, 0x15, 0x54, 0x20*/); 
    /*d0=0x08;*/   d1=0x00;     d2=0x00;   /*d3=0x20;*/ dblSetBuiltinOSD(/*0x08, 0x00, 0x00, 0x20*/); 
      d0=0x02;   /*d1=0x00;*/ /*d2=0x00;*/   d3=0x80;   dblSetBuiltinOSD(/*0x02, 0x00, 0x00, 0x80*/); 
    /*d0=0x02;*/ /*d1=0x00;*/ /*d2=0x00;*/ /*d3=0x80;*/ dblSetBuiltinOSD(/*0x02, 0x00, 0x00, 0x80*/); 
    /*d0=0x02;*/ /*d1=0x00;*/ /*d2=0x00;*/ /*d3=0x80;*/ dblSetBuiltinOSD(/*0x02, 0x00, 0x00, 0x80*/); 
      d0=0x00;     d1=0x80;     d2=0x02;     d3=0x00;   dblSetBuiltinOSD(/*0x00, 0x80, 0x02, 0x00*/); 
    /*d0=0x00;*/   d1=0x20;     d2=0x08;   /*d3=0x00;*/ dblSetBuiltinOSD(/*0x00, 0x20, 0x08, 0x00*/); 
    /*d0=0x00;*/   d1=0x0a;     d2=0xa0;   /*d3=0x00;*/ dblSetBuiltinOSD(/*0x00, 0x0a, 0xa0, 0x00*/); 
    /*d0=0x00;*/   d1=0x00;     d2=0x00;   /*d3=0x00;*/ dblSetBuiltinOSD(/*0x00, 0x00, 0x00, 0x00*/); 
    /*d0=0x00;*/ /*d1=0x00;*/ /*d2=0x00;*/ /*d3=0x00;*/ dblSetBuiltinOSD(/*0x00, 0x00, 0x00, 0x00*/); 
    /* Icon */ 
      d0=0xa5;     d1=0x7f;     d2=0xea;     d3=0xaf;   dblSetBuiltinOSD(/*0xa5, 0x7f, 0xea, 0xaf*/); 
      d0=0xa6;     d1=0x5f;   /*d2=0xea;*/ /*d3=0xaf;*/ dblSetBuiltinOSD(/*0xa6, 0x5f, 0xea, 0xaf*/); 
    /*d0=0xa6;*/   d1=0x9f;   /*d2=0xea;*/ /*d3=0xaf;*/ dblSetBuiltinOSD(/*0xa6, 0x9f, 0xea, 0xaf*/); 
      d0=0x96;     d1=0x97;     d2=0xfa;   /*d3=0xaf;*/ dblSetBuiltinOSD(/*0x96, 0x97, 0xfa, 0xaf*/); 
    /*d0=0x96;*/ /*d1=0x97;*/ /*d2=0xfa;*/ /*d3=0xaf;*/ dblSetBuiltinOSD(/*0x96, 0x97, 0xfa, 0xaf*/); 
    /*d0=0x96;*/ /*d1=0x97;*/ /*d2=0xfa;*/   d3=0xbf;   dblSetBuiltinOSD(/*0x96, 0x97, 0xfa, 0xbf*/); 
    /*d0=0x96;*/   d1=0x96;     d2=0xfe;     d3=0xfe;   dblSetBuiltinOSD(/*0x96, 0x96, 0xfe, 0xfe*/); 
    /*d0=0x96;*/ /*d1=0x96;*/ /*d2=0xfe;*/ /*d3=0xfe;*/ dblSetBuiltinOSD(/*0x96, 0x96, 0xfe, 0xfe*/); 
      d0=0x95;     d1=0x56;   /*d2=0xfe;*/ /*d3=0xfe;*/ dblSetBuiltinOSD(/*0x95, 0x56, 0xfe, 0xfe*/); 
      d0=0x55;     d1=0x55;   /*d2=0xfe;*/ /*d3=0xfe;*/ dblSetBuiltinOSD(/*0x55, 0x55, 0xfe, 0xfe*/); 
      d0=0x56;     d1=0x95;   /*d2=0xfe;*/ /*d3=0xfe;*/ dblSetBuiltinOSD(/*0x56, 0x95, 0xfe, 0xfe*/); 
      d0=0x5a;     d1=0xa5;   /*d2=0xfe;*/ /*d3=0xfe;*/ dblSetBuiltinOSD(/*0x5a, 0xa5, 0xfe, 0xfe*/); 
    /*d0=0x5a;*/ /*d1=0xa5;*/   d2=0xbf;     d3=0xfa;   dblSetBuiltinOSD(/*0x5a, 0xa5, 0xbf, 0xfa*/); 
    /*d0=0x5a;*/ /*d1=0xa5;*/   d2=0xaf;   /*d3=0xfa;*/ dblSetBuiltinOSD(/*0x5a, 0xa5, 0xaf, 0xfa*/); 
    /*d0=0x5a;*/ /*d1=0xa5;*/   d2=0xab;     d3=0xea;   dblSetBuiltinOSD(/*0x5a, 0xa5, 0xab, 0xea*/); 
    /*d0=0x5a;*/ /*d1=0xa5;*/   d2=0x6b;   /*d3=0xea;*/ dblSetBuiltinOSD(/*0x5a, 0xa5, 0x6b, 0xea*/); 
} 

/*
 * turn lut on/off
 */
void OnLut(void)
{  
    value = alGetRegister(0x10);

    if (lut == 0)   /* averlogic lut */
    {
        lut = 1;
        /* R, G, B boot */
        value |= 0x1c;  //set bit 4, 3, 2 
    }
    else if (lut == 1) /* by pass */ 
    {           
        lut = 0;
        value &= ~0x1c;  /* clear bit 4,3,2 */
    }                          
    
    alSetRegister(0x10, value);
}

void dblBootLut(BYTE R, BYTE G, BYTE B)
{
    value = alGetRegister(0x10);
    alSetRegister(0x10, value &= ~0x03);
#if 0    
    dlSinBootLut((float)R);
        
    /* Green */
    alSetRegister(0x10, value | 0x01);
    dlSinBootLut((float)G);
        
    /* Blue */
    alSetRegister(0x10, value | 0x02);
    dlSinBootLut((float)B);  
#else
    dlSinBootLut(R);
        
    /* Green */
    alSetRegister(0x10, value | 0x01);
    dlSinBootLut(G);
        
    /* Blue */
    alSetRegister(0x10, value | 0x02);
    dlSinBootLut(B);  
#endif    
}

#if 0  
void dlSinBootLut(float fboot)
{
    BYTE    n, c;
    float   fColor, fBase;
    float   fDegree;
    
    fBase = 0.7071; /* sin(45 * 0.0174532) */            
    fDegree = 0.7854; /* 45 * 0.0174532 */
    fboot /= 0.2929;  /* 1.0 - 0.7071 */
    
    n = 0;
    while (1)
    {
        fColor = sin(fDegree) - fBase;            
        c = (BYTE)(fColor * fboot) + n;
            
        alSetRegister(0x11, n);
        alSetRegister(0x13, c);
        
        if (n == 255)
            return;
        fDegree += 0.00615;     /* 0.0174532 * 90.0 / 255.0 */                     
        n++;            
    }
}    
#else
#define BRIGHTNESS  16
void dlSinBootLut(BYTE boot)
{
    BYTE    n, c;
    float   fDegree;
    
    fDegree = 0.0; /*1.570788*/; /* 90 * 0.0174532 */
    n = 0;
    while (1)
    {
        if (n <= (256 - BRIGHTNESS))
        {
            c = (BYTE)(sin(fDegree) * boot) + n;
            if (c >= (256 - BRIGHTNESS))
                c = 255;
            else
                c += BRIGHTNESS;            
        }            
        else
            c = 255;
            
        alSetRegister(0x11, n);
        alSetRegister(0x13, c);
        
        if (n == 255)
        {
            alSetRegister(0x11, 0);
            alSetRegister(0x13, 0);
            return;
        }            
        fDegree += 0.013199;     /* 0.0174532 * 180.0 / 255.0 */                     
        n++;            
    }
}    
#endif

void dblSetBuiltinOSD(void /*BYTE d0, BYTE d1, BYTE d2, BYTE d3*/)
{
    alSetRegister(0x11, osd_index++);
    alSetRegister(0x13, d0);
    alSetRegister(0x11, osd_index++);
    alSetRegister(0x13, d1);
    alSetRegister(0x11, osd_index++);
    alSetRegister(0x13, d2);
    alSetRegister(0x11, osd_index++);
    alSetRegister(0x13, d3);
}                              

void OnBorder(void)
{
    crop_state++;                                          
    if (crop_state >= 1 && crop_state <= 3)
    {           
        /* Border color */
        dblSetBorderColor(crop_state == 1 ? 255 : 0, 
                          crop_state == 2 ? 255 : 0, 
                          crop_state == 3 ? 255 : 0);
    }
    else if (crop_state == 4)
    {
        /* Border color */
        dblSetBorderColor(255, 255, 0);
        dblSetSystem(video_system, 4);
    }
    else if (crop_state == 5)
    {
        /* Border color */
        dblSetBorderColor(0, 0, 0);
        dblSetSystem(video_system, (char)-2);
    }        
    else
    {
        dblSetSystem(video_system, 0);
        crop_state = 0;
    }
}          

void OnBootLut(void)
{
    if (bootlut == 1)
    {
        dblBootLut(72, 44, 52);
        bootlut = 0;
    }
    else
    {
        dblBootLut(RED_BOOT, GREEN_BOOT, BLUE_BOOT);
        bootlut = 1;
    }
} 

