#include "decoder.h"
#include "i2c.h"

#if defined(SAA7110)
#define SLAVE_ID       0x9c    /*Slave address to write dac*/
#endif

#if defined(SAA7111)
#define SLAVE_ID    0x48
#endif

// KS0127
#if defined(KS0127)
#define SLAVE_ID         0xD8
#endif

void dcSetDac(unsigned char /*sub_addr*/, unsigned char /*dacData*/);

void dcInitialize()
{
#if defined(SAA7110)
    BYTE i;
    BYTE dac[] = {0x4c, 0x3c, 0x0d, 0xef, 0xbd, 0xf0, 0x40, 0x00,
         0xf8, 0xf8, 0x90, 0x90, 0x00, 0x02, 0x18, 0xf7,
         0x01, 0x2c, 0x5f, 0x40, 0x3b, 0x10, 0xfc, 0xd2,
         0xf0, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
         0xd9, 0x16, 0x40, 0x40, 0x80, 0x40, 0x80, 0x4f,
         0xfe, 0x01, 0xcf, 0x0f, 0x03, 0x01, 0x81, 0x0a,
         0x44, 0x35, 0x02, 0x8c, 0x03};

    for (i=0; i<= 0x34; i++)
    {
        dcSetDac(i, dac[i]);
    } 
#endif

#if defined(SAA7111)
    dcSetDac(0x02, 0xc0);
    dcSetDac(0x03, 0x23);
    dcSetDac(0x04, 0x00);
    dcSetDac(0x05, 0x00);
    dcSetDac(0x06, 0xeb);
    dcSetDac(0x07, 0xe0);
    dcSetDac(0x08, 0x88);    /* VTR input */
    dcSetDac(0x09, 0x41);
    dcSetDac(0x0a, 0x80);
    dcSetDac(0x0b, 0x4c);
    dcSetDac(0x0c, 0x40);    /* CCIR YUV422 */
    dcSetDac(0x0d, 0x00);
    dcSetDac(0x0e, 0x01);
    dcSetDac(0x10, 0x40);
    dcSetDac(0x11, 0x1c);
    dcSetDac(0x12, 0x01);
#endif

#if defined(KS0127)
    BYTE i;
    BYTE dac[] = {
                0x5C, 0x22, 0x40, 0x00, 0x00, 0x00, 0x14, /* 01 ~ 07 */
          0x00, 0x00, 0x00, 0x84, 0x00, 0x00, 0x00, 0x01, /* 08 ~ 0F */
          0x00, 0x00, 0x0A, 0x00, 0x80, 0x00, 0x00, 0x00, /* 10 ~ 17 */
          0x04, 0x0C, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 18 ~ 1F */
          0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, /* 20 ~ 27 */
          0x00, 0x00, 0x33, 0x00, 0x00, 0x0B, 0x00, 0x00, /* 28 ~ 2F */
          0x41, 0x00, 0x10, 0x80, 0x80, 0x10, 0x80, 0x80, /* 30 ~ 37 */
          0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04  /* 38 ~ 3F */
    };                
    
    for (i=0; i<0x3f; i++)
    {
        dcSetDac(i+1, dac[i]);
    }
#endif
}

void dcSetSystem(BYTE system)
{
#if defined(SAA7110)
    if (system == 0) /* NTSC */
    {
        dcSetDac(0x11, 0x2c);
        dcSetDac(0x0f, 0xf7);
    }
    else /* PAL */
    {
        dcSetDac(0x11, 0x59);
        dcSetDac(0x0f, 0xb7);
    }
#endif

#if defined(SAA7111)
    /* auto detection */
#endif

#if defined(KS0127)
    /* auto detection */
#endif
}

void dcSetVideoSource(BYTE source)
{
#if defined(SAA7110)
    if (source == 0) /* Composite */
    {
        dcSetDac(0x20, 0xd9);
        dcSetDac(0x21, 0x16);
        dcSetDac(0x22, 0x40);
        dcSetDac(0x2c, 0x03);
        dcSetDac(0x06, 0x40);
        /*dcSetDac(0x30, 0x44);*/
    }
    else if (source == 1) /* svideo */
    {
        dcSetDac(0x20, 0x3c);
        dcSetDac(0x21, 0x21);
        dcSetDac(0x22, 0xc1);
        dcSetDac(0x2c, 0x23);
        dcSetDac(0x06, 0xc0);
        /*dcSetDac(0x30, 0x44);*/
    }
#endif

#if defined(SAA7111)
    if (source == 0) /* Composite */
    {
        dcSetDac(0x02, 0xc0);
        dcSetDac(0x09, 0x41);
    }
    else if (source == 1) /* svideo */
    {
        dcSetDac(0x02, 0xc5);
        dcSetDac(0x09, 0x81);
    }
#endif

#if defined(KS0127)
    if (source == 0) /* Composite */
    {
        dcSetDac(0x02, 0x22);
    }
    else if (source == 1) /* svideo */
    {
        dcSetDac(0x02, 0x28);
    }
#endif
}

void dcSetDac(unsigned char sub_addr, unsigned char dacData)
{
    i2cWrite(SLAVE_ID, sub_addr, dacData);
}

unsigned char dcAutoDetect(void)
{
#if defined(SAA7111)
    return((i2cRead(SLAVE_ID, (BYTE)0x1f) & 0x20) ? 1 : 0);
#elif defined(KS0127)
    return((i2cRead(SLAVE_ID, (BYTE)0x00) & 0x10) ? 1 : 0);
#else
    return((unsigned char)0xff);	/* not supported */
#endif
}


