/*----------------------------------------------------------------------------
 *  Copyright 1996,1997     Averlogic Technologies Inc.
 *----------------------------------------------------------------------------*/

#include <reg51.h>
#include "i2c.h"

#define MAX_RETRY   3

#define FALSE   0
#define TRUE    1

typedef char BOOL; 
typedef unsigned char BYTE, *PBYTE;

/*
 * Functions provided by Master chip producer
 */ 
/*void i2cEnable();*/
/*void i2cDisable();*/
                                             
/*
 * Local functions                                             
 */
BOOL i2cLocate(BYTE slave_addr, BYTE  sub_addr);
void i2cStart(void);
void i2cStop(void);
void i2cTransmit(BYTE /*value*/);
BOOL i2cWaitAck(void);
void i2cNotAck(void);
/*void i2cGiveAck();*/
BYTE i2cReceive(void);
void i2cSetSCL1(void);
void i2cSetSCL0(void);
void i2cSetSDA1(void);
void i2cSetSDA0(void);
BYTE i2cGetSDA(void);
void i2cReleaseSDA(void);

#if 1
sbit _scl = P1^0;
sbit _sda = P1^1;
#else
_sfrbit _scl _atbit(P1, 0);
_sfrbit _sda _atbit(P1, 1);
#endif

/*
 *Exported functions
 */
BOOL i2cWrite(BYTE slave_addr, BYTE  sub_addr, BYTE value)
{
    if (i2cLocate(slave_addr, sub_addr))
    {
        /*value */
        i2cTransmit(value);
        if (i2cWaitAck() == 0)
        {    
            i2cStop();   
            return(FALSE);
        }        
              
        /*Stop I2C            */
        i2cStop();   
             
        return(TRUE);     
    }
}

unsigned char i2cRead(BYTE slave_addr, BYTE  sub_addr)
{      
    unsigned char value;
    
    if (i2cLocate(slave_addr,  sub_addr))
    {
        //Read data byte pointed                            
        //I2C Start                 
        i2cStart();

        i2cTransmit((BYTE)(slave_addr | 0x0001));  /*read*/
        if (!i2cWaitAck())
        {    
            i2cStop();
            return(0xff);
        }        
        value = i2cReceive();
        i2cNotAck();
               
        i2cStop();
        
        /*Succeed!             */
        return(value);     
    }
}

BOOL i2cLocate(BYTE slave_addr, BYTE  sub_addr)
{
    BYTE i;
                     
    for (i=0; i<MAX_RETRY; i++)
    {    
        /*Start I2C */             
        i2cStart();
    
        /*Slave address    */
        i2cTransmit((BYTE)slave_addr);
        if (i2cWaitAck() == 0)
        {    
            i2cStop();   
            continue;
        }        
                       
        /*Subaddress         */          
        i2cTransmit((BYTE)sub_addr);
        if (i2cWaitAck() == 0)
        {    
            i2cStop();   
            continue;
        }        
                    
        return(TRUE);     
    }

    /* Fail */    
    return(FALSE);     
} 

/*
 *Local functions
 */
void i2cStart(void)
{
    i2cSetSCL0();
    i2cSetSDA1();
    i2cSetSCL1();
    i2cSetSDA0();
    i2cSetSCL1();
}

void i2cStop(void)
{
    i2cSetSCL0();
    i2cSetSDA0();
    i2cSetSCL1();
    i2cSetSDA1();
    i2cSetSCL1();
    i2cSetSCL0();
}

void i2cTransmit(BYTE value)
{
    BYTE i;

    /*Set value bit by bit...MSB first*/
    for (i=0; i<8; i++)
    {                                                        
        i2cSetSCL0();
        if (value & 0x80)
            i2cSetSDA1();
        else
            i2cSetSDA0();
        i2cSetSCL1();
        
        value <<= 1;
    }
    i2cSetSCL0();
    
    i2cReleaseSDA();
}
                         
#define MAX_WAIT_COUNT  254                         
BOOL i2cWaitAck(void)
{
    BYTE   count = 0L;
        
    /*transmit state, ack set by receiver*/
    i2cSetSCL0();

    while (i2cGetSDA() && count++ <= MAX_WAIT_COUNT);
    
    if (i2cGetSDA())
        return(FALSE);

    i2cSetSDA0();
    i2cSetSCL1();
    i2cSetSCL0();
    return(TRUE);
}

void i2cNotAck(void)
{
    /*receive state, ack set by master*/
    i2cSetSCL0();
    i2cSetSDA1();
    i2cSetSCL1();
    i2cSetSCL0();
}

BYTE i2cReceive(void)
{
    /*Read ack.*/  
    BYTE     i;
    BYTE    value;                                                     
    
    i2cReleaseSDA();

    /*Set value bit by bit...MSB first*/
    value = 0;
    for (i=0; i<8; i++)
    {                                                        
        value <<= 1;
        i2cSetSCL0();
        value |= i2cGetSDA();               
        i2cSetSCL1();
    }
    i2cSetSCL0();

    return(value);    
}


void i2cSetSCL1(void)
{
    _scl = 1;
    /*_scl = 1;*/
}

void i2cSetSCL0(void)
{
    _scl = 0;
    /*_scl = 0;*/
}

void i2cSetSDA1(void)
{
    _sda = 1;
    /*_sda = 1;*/
}

void i2cSetSDA0(void)
{
    _sda = 0;
    /*_sda = 0;*/
}

BYTE i2cGetSDA(void)
{
    if (_sda == 0)
        return(0);
    return(1);    
}    

void i2cReleaseSDA(void)
{
    _sda = 1;   /*pull high*/ 
}    

/*
void i2cEnable(void)
{
}

void i2cDisable(void)
{
}    
*/