#if 1
#include <reg51.h>
#else
#include <reg51.sfr>
#endif
#include "i2c.h"
#include "decoder.h"
#include "al250.h"  

/*
 * Button definition
 */
#if 1
sbit _source      = P3^0;
sbit _system      = P3^1;
sbit _lut         = P3^3;
sbit _osd         = P3^2;
sbit _overlay     = P3^4;
sbit _border      = P3^5;
sbit _bootlut     = P3^6;
sbit _home        = P3^7;
#else
_sfrbit _system      _atbit( P3, 7 );
_sfrbit _source      _atbit( P3, 6 );
_sfrbit _osd         _atbit( P3, 1 );
_sfrbit _overlay     _atbit( P3, 0 );

_sfrbit _ovl0      _atbit( P1, 2 );
_sfrbit _ovl1      _atbit( P1, 3 );
_sfrbit _ovl2      _atbit( P1, 4 );
_sfrbit _ovl3      _atbit( P1, 5 );
#endif  /* 1 */

BYTE system = 0;    /* NTSC */
BYTE source = 0;    /* Composite */
                                
#define BUTTON_IDLE     0                                
#define BUTTON_ACTIVE   1
#define BUTTON_INIT     1

unsigned char alGetRegister(unsigned char index);
void alSetRegister(unsigned char index, unsigned char value);
void alDelay(unsigned int nMS);

void main()
{
    alDelay(300);    /* wait decoder to reset */
    dblInitialize();
    
    dcInitialize();    /* NTSC, Composite */
    dcInitialize();    /* NTSC, Composite */
    /*dcSetSystem(system);*/
    /*dcSetVideoSource(source);*/
    
    /*dblSetSystem(system, 0);*/
    while (1)
    {
        P3 = 0xff;     /*pull high*/
        alDelay(20);
        if (system != dblAutoDetect())
        {
            system = 1 - system;
            dblSetSystem(system, 0);
        }            
        if (_system == BUTTON_ACTIVE)
        {   
            system = 1 - system;
            dcSetSystem(system);
        }
        else if (_source == BUTTON_ACTIVE)
        {
            source = 1 - source;
            dcSetVideoSource(source); 
        }
        else if (_osd == BUTTON_ACTIVE)
        {
            OnBuiltinOSD();
        }
        else if (_overlay == BUTTON_ACTIVE)
        {
            OnExternalOverlay();
        }
        else if (_lut == BUTTON_ACTIVE)
        {    
            OnLut();
        }  
        else if (_border == BUTTON_ACTIVE)
        {    
            OnBorder();
        }  
        else if (_bootlut == BUTTON_ACTIVE)
        {    
            OnBootLut();
        }  
        else if (_home == BUTTON_ACTIVE)
        {    
            dblHome();
        }  
        
        while (P3 != BUTTON_IDLE);
    }
}

/*
 * I/O
 */ 

#define AL250_SLAVE_ID  0x58
unsigned char alGetRegister(unsigned char index)
{
    return(i2cRead((unsigned char)AL250_SLAVE_ID, index));
}

void alSetRegister(unsigned char index, unsigned char value)
{
    i2cWrite(AL250_SLAVE_ID, index, value);
} 
  
/*  
 * Timer
 */
void alDelay(unsigned int nMS)
{
    TMOD = 1;  /*mode 1, 16-bit counter*/
    TR0 = 0;   /*stop timer*/
    TF0 = 0;
    while (nMS != 0)
    {
        TH0 = 252;      /*65536-1000 / 256*/
        TL0 = 24;     /*65536-1000 % 256*/
        TR0 = 1;   /*start timer*/
        while (!TF0);  /*check overflow*/
        TR0 = 0;   /*stop timer*/
        TF0 = 0;   
        nMS -= 1;
    }        
}
